// The ppLimbScan macro traces a series of rays to scan an object and generate a pair of arrays containing
// coordinates of points on the surface of the object and the normals at those points.
// The macro moves along a straight line from ppStartPos to ppEndPos sending out a ring of rays to scan the
// surface on a plane at right angles to the vector as it transitions from the direction of the start vector to
// the position of the end vector. ppSlices tells the macro how many points to break
// the line into and ppSegments tells the macro how many rays to send out from each point.
// The results are written back to a pair of arrays that have to be defined prior to calling the macro.
//
// Parameters:
// -----------
//     ppObject           - The object to be scanned
//     ppStartPos         - A point inside the object where the scan will start
//     ppStartVector      - The normal of a surface on which rays will be traced out from the start point
//     ppStartRefVector   - This reference vector helps keep the start of the circular sweep of each slice pointing 
//                          in a predictable direction across multiple calls to this macro
//     ppEndPos           - A point inside the object where the scan will end
//     ppEndVector        - The normal of a surface on which rays will be traced out from the end point
//     ppEndRefVector     - This reference vector helps keep the start of the circular sweep of each slice pointing 
//                          in a predictable direction across multiple calls to this macro
//     ppSlices           - The number of slices to be taken along the line from the start point to the end point
//     ppSegments         - The number of radial rays to be sent out from each point
//     ppObjectArray      - The array that will be used to store the points detected on the surface of the object
//     ppNormalArray      - The array to be used to store the normals at those points
//     ppArrayStart       - The starting number of the first dimension to use in the arrays (often '0')
//     ppDirection        - -1 or +1 for positive or negative direction of rotation of the trace 
//     ppTraceLength      - The distance from the centre from which the ray will be traced back in towards the centre 

#macro ppLimbScan (ppObject, ppStartPos, ppStartVector, ppStartRefVector, ppEndPos, ppEndVector, ppEndRefVector, ppSlices, ppSegments, ppObjectArray, ppNormalArray, ppArrayStart, ppDirection, ppTraceLength)
  #local ppSeed = seed(1);
  #if ( ppVerbose = 1) #debug concat("ppLimbScan Macro Call. \n") #end
  #if (vlength(ppStartVector)=0)  #local ppStartVector = vnormalize(ppEndPos-ppStartPos);  #end
  #if (vlength(ppEndVector)=0)    #local ppEndVector   = vnormalize(ppEndPos-ppStartPos);  #end
  #local ppI = 0; 
  #while (ppI<ppSlices)
    #local ppRefVector = ppStartRefVector + ppI*((ppEndRefVector-ppStartRefVector)/(ppSlices-1));
    #local ppSliceCentre = ppStartPos + (ppEndPos-ppStartPos)*(ppI/(ppSlices-1));
    #local ppSliceVector = ppStartVector + ppI*((ppEndVector-ppStartVector)/(ppSlices-1));

    #if (ppVerbose = 1)
      #debug concat("Slice Vector ",str(ppI,3,0),": <",vstr(3,ppSliceVector,",",0,3),">\n")
      #debug concat("VPerp of Slice Vector ",str(ppI,3,0),": <",vstr(3,VPerp_Adjust(ppRefVector, ppSliceVector),",",0,3),">\n")
    #end
    #if ((ppI = ppSlices-1 | ppI = 0) & ppVerbose = 1)
      #debug concat("Slice Centre: <",vstr(3,ppSliceCentre,",",0,3),">")
      #debug concat("Start Vector: <",vstr(3,ppStartVector,",",0,3),">")
      #debug concat("End Vector  : <",vstr(3,ppEndVector,",",0,3),">","\n")
    #end

    #local ppJ = 0;
    #while (ppJ<ppSegments)
      #local ppSliceAngle = -ppDirection*ppJ*360/ppSegments; 
      #local ppNorm = <0, 0, 0>;
      #local ppRayDirection = vaxis_rotate(VPerp_Adjust(ppRefVector, ppSliceVector),ppSliceVector,ppSliceAngle); // VPerp could be problem. 
      #local ppRayDirection = vnormalize(ppRayDirection); 
      #local ppThisPoint = trace(ppObject, 0.0001+ppSliceCentre+(ppRayDirection*ppTraceLength), -ppRayDirection, ppNorm);
      #if (vlength(ppNorm)!=0) 

//#if (ppShowPoints & ppJ=11 & ppI = ppSlices-2)
////cylinder {ppSliceCentre,ppSliceCentre+VPerp_Adjust(ppRefVector, ppSliceVector)  *30 ,0.002 pigment {color rgb <2,0,2>}}
////cylinder {ppSliceCentre,ppSliceCentre+ppRefVector  *30 ,0.002 pigment {color rgb <2,2,0>}}
////cylinder {ppSliceCentre,ppSliceCentre+ppSliceVector*30 ,0.002 pigment {color rgb <0,2,2>}}
//
//difference {
//object {ppObject pigment {color rgbt <1,1,1,0.5>}}
//plane {-y,0 translate ppSliceCentre pigment {color rgbt <1,1,1,0.9>}}
//}
//cylinder {ppSliceCentre,ppSliceCentre+ppRayDirection*30 ,0.002 pigment {color rgb <2,2,2>}}
//#end 
//#if (ppShowPoints & ppI = ppSlices-2)
//sphere {ppThisPoint,0.005 pigment {color rgb <0,2,2>}}
//#end 
//#if (ppShowPoints & ppJ = 10)
//sphere {ppThisPoint,0.005 pigment {color rgb <2,0,2>}}
//#end 

        #local ppObjectArray[ppI+ppArrayStart][ppJ] = ppThisPoint;
        #local ppNormalArray[ppI+ppArrayStart][ppJ] = ppRayDirection; //ppNorm;
      #else
//        #local ppObjectArray[ppI+ppArrayStart][ppJ] = <0.1*rand(ppSeed),2+0.1*rand(ppSeed),0.1*rand(ppSeed)>;
        #local ppObjectArray[ppI+ppArrayStart][ppJ] = <0,1,0>;
        #local ppNormalArray[ppI+ppArrayStart][ppJ] = <0,1,0>;
        #warning concat("The ppLimbScan Macro sent out a ray from point <",vstr(3,ppSliceCentre,",",0,3),"> that didn't intercept the object.\n")
      #end        
      #local ppJ = ppJ + 1;
    #end
    #local ppI= ppI + 1;
  #end
#end  

// The ppEndScan macro takes an object and a ring of points around the object, then works out a series of centre points 
// (that need to lie inside the object) from which it can send out rays to scan the end of the object.
// It then generates a pair of arrays containing coordinates of points on the surface of the object and the normals at those points.
// The ObjectArray passed to this macro must contain a list of points that track the surface of the object in the dimension where 
// the first index is '0'. The list of points passed in are assumed to lie in a plane. 
//
// Parameters:
// -----------
//     ppObject           - The object to be scanned  
//     ppStartVector      - The normal of the surface passed in via the '0' elements of ppObjectArray 
//     ppSlices           - The number of points to be taken along the curve from the initial points to point where the surfaces meet
//     ppSegments         - The size of the second dimension of the ppObjectArray
//     ppObjectArray      - The array that will be used to store the points detected on the surface of the object
//     ppNormalArray      - The array to be used to store the normals at those points
//     ppDirection        - -1 or +1 for positive or negative direction of rotation of the trace 
//     ppTraceLength      - The distance from the centre from which the ray will be traced back in towards the centre
//     ppScanMethod       - 1=Diagonal to opposite point: 2=Parallel, with '0' element in middle of long side 
//
#macro ppEndScan (ppObject, ppStartVector, ppSlices, ppSegments, ppObjectArray, ppNormalArray, ppDirection, ppTraceLength, ppScanMethod)
  // Take 3 points and work out the normal to the plane of the existing slice through the object
  #local ppPlaneNormal = VPerp_To_Plane(ppObjectArray[0][1]-ppObjectArray[0][0],ppObjectArray[0][0]-ppObjectArray[0][ppSegments/2]); 
  #local ppIntegerPart = div(ppSegments,4);
  #local ppRemainder = ppSegments-(ppIntegerPart*4);
  #local ppTipOffset = -1;
  #if (ppRemainder=0) #local ppTipOffset = div(ppSegments,4);   #end  
  #if (ppRemainder=3) #local ppTipOffset = div(ppSegments,4)+1; #end  
  #if (ppScanMethod=1)
    #local ppJ = 0;
    #local ppTotalVector = <0,0,0>;
    #while (ppJ<ppSegments)
      #local ppTotalVector = ppTotalVector + ppObjectArray[0][ppJ];
      #local ppJ = ppJ + 1;
    #end
    #local ppCentrePoint = ppTotalVector/ppSegments;     // (  *** Might be better to simply pass this in as a parameter ***  )
  #end
  #local ppJ = 0;
  #while (ppJ<ppSegments)  
    #if (ppScanMethod=2) 
      #local ppDone = 0;
      #if (ppJ=0)
        #if(ppRemainder=0|ppRemainder=2)
          #local ppCentrePoint = (ppObjectArray[0][0]+ppObjectArray[0][div(ppSegments,2)])/2;
          #local ppDone=1;

//cylinder {ppObjectArray[0][10],ppObjectArray[0][10]+ppPlaneNormal*10,0.002 pigment {color rgb <1,0,0>}}
//cylinder {ppCentrePoint,ppCentrePoint+ppPlaneNormal*10,0.002 pigment {color rgb <1,0,0>}}
        #end
        #if(ppRemainder=1|ppRemainder=3)
          #local ppCentrePoint = (ppObjectArray[0][0]+((ppObjectArray[0][div(ppSegments,2)] + ppObjectArray[0][div(ppSegments,2)+1])/2))/2;
          #local ppDone=1;
        #end 
      #end 
      #if ((ppRemainder=0|ppRemainder=3)&(ppJ=ppTipOffset))
        #local ppCentrePoint = (ppObjectArray[0][ppTipOffset-1]+ppObjectArray[0][ppTipOffset+1])/2;  
        #local ppDone=1;
      #end
      #if ((ppRemainder=0|ppRemainder=3)&(ppJ=ppSegments-ppTipOffset))
        #local ppCentrePoint = (ppObjectArray[0][ppSegments-ppTipOffset-1]+ppObjectArray[0][ppSegments-ppTipOffset+1])/2;  
        #local ppDone=1;
      #end
      #if (ppDone=0)
        #if (ppJ <= (ppSegments/2)) #local ppK = div(ppSegments+1,2)-ppJ;
        #else                       #local ppK = div(ppSegments,2)+ppSegments-ppJ;
        #end
        #local ppCentrePoint = (ppObjectArray[0][ppJ]+ppObjectArray[0][ppK])/2;  
//cylinder {ppCentrePoint,ppCentrePoint+ppPlaneNormal*10,0.001 pigment {color rgb <1,1,0>}}
      #end
    #end
    #local ppI = 1;
    #while (ppI<ppSlices) 
      #local ppSliceAngle  = ppDirection*ppI*90/(ppSlices-1);
      #local ppSliceVector = VPerp_To_Plane(ppPlaneNormal,ppObjectArray[0][ppJ]-ppCentrePoint); 
      #local ppRayDirection = vaxis_rotate(ppPlaneNormal,ppSliceVector,90-ppSliceAngle); 
      #local ppRayDirection = vnormalize(ppRayDirection); 
      #local ppNorm = <0, 0, 0>;
      #local ppThisPoint = trace(ppObject, ppCentrePoint+(ppRayDirection*ppTraceLength), -ppRayDirection, ppNorm);
      #if (vlength(ppNorm)!=0) 
        #local ppObjectArray[ppI][ppJ] = ppThisPoint;
        #local ppNormalArray[ppI][ppJ] = ppNorm;
      #else
        #local ppObjectArray[ppI][ppJ] = <0,0,0>;
        #local ppNormalArray[ppI][ppJ] = <0,1,0>;
        #warning concat("The ppEndScan Macro sent out a ray from point <",vstr(3,ppCentrePoint,",",0,3),"> that didn't intercept the object.\n")
      #end        
      #local ppI = ppI + 1;
    #end
    #local ppJ = ppJ + 1;
  #end
#end

// Add an offset to a 2D array of positions pushing each position out in the direction of the corresponding normal. 
// The offset is picked up from a combination of the value of a pigment at a corresponding point and a fixed offset. 
// The proportion of the pigment to take into account can be controlled using the ppAmount parameter. If set to zero, 
// the positions are consistently displaced by the amount specified using just the ppOffSet parameter.
//
// Parameters:
// -----------
//     ppIStart           - The start of the first dimension of the array
//     ppIEnd             - The end of the first dimension of the array
//     ppJStart           - The start of the second dimension of the array
//     ppJEnd             - The end of the second dimension of the array  
//     ppObjectArray      - The array containing the positions of the points
//     ppNormalArray      - The array containing the surface normals
//     ppPigment          - The pigment to use to specify the offset for the corresponding points
//     ppAmount           - A multiplication factor to use for the pigment
//     ppOffSet           - An offset to add, irrespective of the pigment value 
//
#macro ppAddOffSetToArray (ppIStart, ppIEnd, ppJStart, ppJEnd, ppObjectArray, ppNormalArray, ppPigment,ppAmount,ppOffSet)
  #if ( ppVerbose = 1) #debug concat("ppAddoffSetToArray Macro Call. \n") #end
  #local ppI = ppIStart;
  #while (ppI<=ppIEnd)
    #local ppJ = ppJStart;
    #while (ppJ<=ppJEnd)
      #local ppThisPoint = ppObjectArray[ppI][ppJ];
      #local ppThisPigment = eval_pigment(ppPigment, ppThisPoint);
      #local ppObjectArray[ppI][ppJ] = ppThisPoint+ppNormalArray[ppI][ppJ]*(ppAmount*ppThisPigment.x+ppOffSet);
      #local ppJ = ppJ + 1;
    #end
    #local ppI= ppI + 1;
  #end
#end


// The ppStretchPoints macro fixes a difficulty with a particular use of the ppLimbScan macro. There are circumstances where
// it is easier to slice a bit out of an object and scan right over the cut. However, when you do this you can end up with 
// sharp edges on the object and sections of the clothing wrapped around such an edge are likely to intercept the object.
// This macro detects points that are outside the surface of a plane used to slice an object and moves them outwards so that
// they all lie on the surface of the object.   
//
// Parameters:
// -----------
//     ppObject           - The object used to generate the array of points (using the ppLimbScan macro) 
//     ppArray            - The array of points
//     ppIStart           - The beginning of the first array dimension to process 
//     ppIEnd             - The end of the first array dimension to process
//     ppJStart           - The beginning of the second array dimension to process
//     ppJEnd             - The end of the second array dimension to process      
//     ppPlaneVector      - The orientation of the plane that was used to slice through the object
//     ppPlanePosition    - The position of the plane that was used to slice through the object
//
#macro ppStretchPoints(ppObject, ppObjectArray, ppIStart, ppIEnd, ppJStart, ppJEnd, ppPlaneVector, ppPlanePosition)
  #if ( ppVerbose = 1) #debug concat("ppStretchPoints Macro Call. \n") #end 
  #local ppPlaneObject = plane {ppPlaneVector,0 translate ppPlanePosition};
  #local ppSliceCentre = ppPlanePosition - ppPlaneVector*0.001; // Should be just inside the cut face of the object 
//  #local ppTraceLength = 0.5; 
  #local ppI = ppIStart;
  #while (ppI<=ppIEnd)
    #local ppJ = ppJStart;
    #while (ppJ<=ppJEnd)
      #local ppThisPoint = ppObjectArray[ppI][ppJ];
      
      #if (vlength(VProject_Plane(ppThisPoint-ppPlanePosition, ppPlaneVector)-(ppThisPoint-ppPlanePosition))=0 | inside(ppPlaneObject,ppThisPoint)=0.0)
//        #warning concat("The ppStretchPoints Macro detected point <",vstr(3,ppThisPoint,",",0,3),"> outside the plane.\n")
        #local ppRayDirection = VProject_Plane(ppThisPoint-ppPlanePosition, ppPlaneVector);
        #local ppNorm = <0,0,0>; 
        #local ppThisPoint = trace(ppObject, ppSliceCentre, ppRayDirection, ppNorm);            
//        #local ppThisPoint = trace(ppObject, ppSliceCentre+(ppRayDirection*ppTraceLength), -ppRayDirection, ppNorm);            
        #if (vlength(ppNorm)!=0) 
          #local ppObjectArray[ppI][ppJ] = ppThisPoint;
        #else
          #warning concat("The ppStretchPoints Macro sent out a ray from point <",vstr(3,ppSliceCentre,",",0,3),"> that didn't intercept the object.\n")
        #end  
//        #local ppObjectArray[ppI][ppJ] = <0,0,0>;
      #end
      #local ppJ = ppJ + 1;
    #end
    #local ppI= ppI + 1;
  #end
#end
  
// The ppArrayBand macro adds thickness to a 2D array that contains a series of positions representing a tubular shape. 
// First it makes the first and last band equal to those one in from either end.
// Then, for all but the end two bands it adds an offset to move the points outwards.   
//
// Parameters:
// -----------
//     ppPositionArray    - The 2D array containing the positions of the points 
//     ppNormalArray      - The 2D array containing the corresponding normals of the points
//     ppArray1Dim        - The size of the first dimension of the array
//     ppArray2Dim        - The size of the second dimension of the array
//     ppThickness        - The amount to thicken out the surface represented by the points 
//
#macro ppArrayBand(ppPositionArray, ppNormalArray, ppArray1Dim, ppArray2Dim, ppThickness) 
  #if ( ppVerbose = 1) #debug concat("ppArrayBand Macro Call. \n") #end
  #local ppJ = 0;
  #while (ppJ<ppArray2Dim)
    #local ppPositionArray[0][ppJ] = ppPositionArray[1][ppJ];  
    #local ppNormalArray[0][ppJ]   = ppNormalArray[1][ppJ];  
    #local ppPositionArray[ppArray1Dim-1][ppJ] = ppPositionArray[ppArray1Dim-2][ppJ];  
    #local ppNormalArray[ppArray1Dim-1][ppJ]   = ppNormalArray[ppArray1Dim-2][ppJ];  
    #local ppJ = ppJ + 1;
  #end
  #local ppI = 1;
  #while (ppI < ppArray1Dim-1)  
    #local ppJ = 0;
    #while (ppJ<ppArray2Dim)
      #local ppPositionArray[ppI][ppJ] = ppPositionArray[ppI][ppJ] + ppThickness*vnormalize(ppNormalArray[ppI][ppJ]);
      #local ppJ = ppJ + 1;
    #end
    #local ppI = ppI + 1;                        
  #end
#end

// The ppStitchArrays macro takes a line of points from one pair of 2D arrays containing a set of positions and normals 
// and copies them into a corresponding line in a second pair of 2D arrays. When the two arrays are subsequently 
// used to define a mesh of smooth triangles, the two meshes should join together 'seamlessly'. 
// There are certain conditions that need to be met to help this to happen because, if the edges don't 
// approximately line up before you calling this macro, then stitching them together isn't going to work that well.
//
// Parameters:
// -----------
//     ppPositionArray1   - The first 2D array of positions
//     ppNormalArray1     - The first 2D array of normals
//     ppI1               - The index to the line of points in the first pair of 2D arrays
//     ppPositionArray2   - The second 2D array of positions
//     ppNormalArray2     - The second 2D array of normals  
//     ppI2               - The index to the line of points in the second pair of 2D arrays
//     ppSegments         - The size of the second dimension of the 2D arrays
//
#macro ppStitchArrays(ppPositionArray1,ppNormalArray1, ppI1,ppPositionArray2,ppNormalArray2, ppI2,ppSegments)
  #if ( ppVerbose = 1) #debug concat("ppStitchArrays Macro Call. \n") #end
  #local ppJ = 0;
  #while (ppJ<ppSegments)
    #local ppPositionArray2[ppI2][ppJ] = ppPositionArray1[ppI1][ppJ];
    #local ppNormalArray2[ppI2][ppJ]   = ppNormalArray1[ppI1][ppJ];
    #local ppJ = ppJ + 1;
  #end
#end

// The ppMeshFromArray macro takes an array of Positions and a corresponding array of Normals and generates a mesh 
// object.
//
// Parameters:
// -----------
//     ppArray            - The 2D array of positions
//     ppNormal           - The 2D array of normals
//     ppIDim             - The size of the first dimension
//     ppJDim             - The size of the second dimension  
//     ppJoinEnds         - Whether to join the ends of the mesh together - 1=join them, 0 = leave the mesh open
//
#macro ppMeshFromArray(ppArray,ppNormal,ppIDim,ppJDim,ppJoinEnds)
  #if ( ppVerbose = 1) #debug concat("ppMeshFromArray Macro Call. \n") #end
  #local ppI = 0;
  mesh {
    #while (ppI<ppIDim-1)
      #local ppNextI = ppI+1;
      #local ppJ = 0;  
      #if (ppJoinEnds) #local ppJLimit = ppJDim; 
      #else #local ppJLimit = ppJDim-1; #end
      #while (ppJ<ppJLimit)
        #local ppNextJ = ppJ + 1;
        #if (ppNextJ=ppJDim) #declare ppNextJ = 0; #end
        smooth_triangle {
          ppArray[ppI][ppJ]    ,ppNormal[ppI][ppJ]    
          ppArray[ppI][ppNextJ],ppNormal[ppI][ppNextJ]
          ppArray[ppNextI][ppJ],ppNormal[ppNextI][ppJ]
        }
        smooth_triangle {
          ppArray[ppNextI][ppNextJ],ppNormal[ppNextI][ppNextJ]    
          ppArray[ppI][ppNextJ]    ,ppNormal[ppI][ppNextJ]
          ppArray[ppNextI][ppJ]    ,ppNormal[ppNextI][ppJ]
        }
        #declare ppJ = ppJ + 1;
      #end
      #declare ppI = ppI + 1;
    #end
  }
#end  

// The ppMesh2FromArray macro takes an array of Positions and generates a mesh2 object.  
//
// Parameters:
// -----------
//     ppArray            - The 2D array of positions 
//     ppNormal           - An optional 2D array of normals
//     ppIDim             - The size of the first dimension
//     ppJDim             - The size of the second dimension  
//     ppJoinEnds         - Whether to join the ends of the mesh together - 1=join them, 0 = leave the mesh open
//
#macro ppMesh2FromArray(ppArray,ppNormalArray,ppIDim,ppJDim,ppJoinEnds)
  #if ( ppVerbose = 1) #debug concat("ppMeshFromArray Macro Call. \n") #end
  // Rewritten based on an example posted by Scott on the povray.binaries.scene-files newsgroup.
  #if (ppJoinEnds) #local ppJTotal = ppJDim + 1;
  #else            #local ppJTotal = ppJDim;
  #end
  mesh2{
    vertex_vectors{
      ppIDim*ppJTotal
      #local ppI = 0;
      #while(ppI<ppIDim)
        #local ppJ=0;
        #while(ppJ<ppJTotal) 
//
//#debug concat("I:J = ",str(ppI,3,0),":",str(ppJ,3,0),"\n")
//
//
          #if (ppJ<ppJDim) ppArray[ppI][ppJ]
          #else            ppArray[ppI][0]
          #end
          #local ppJ=ppJ+1;
        #end
        #local ppI=ppI+1;
      #end
    }
    normal_vectors{
      ppIDim*ppJTotal
      #local ppI=0;
      #while(ppI<ppIDim)
        #local ppJ=0;
        #while(ppJ<ppJTotal)
          #ifdef (ppNormalArray[ppI][ppJ]) 
            ppNormalArray[ppI][ppJ]
          #else
          #local ppNormal  = <0,0,0>;
          #local ppNormalCount = 0; 
            #if (ppJoinEnds)
              #if (ppI>=1 & ppJ=ppJDim)
                #local ppNormal = ppNormal + vcross(ppArray[ppI][0]-ppArray[ppI][1],ppArray[ppI][0]-ppArray[ppI-1][0]);
                #local ppNormalCount = ppNormalCount+1;
              #end
              #if (ppI<=ppIDim-2 & ppJ=ppJDim)
                #local ppNormal = ppNormal - vcross(ppArray[ppI][0]-ppArray[ppI][1],ppArray[ppI][0]-ppArray[ppI+1][0]);
                #local ppNormalCount = ppNormalCount+1;    
              #end
              #if (ppI>=1 & ppJ=ppJDim-1)
                #local ppNormal = ppNormal + vcross(ppArray[ppI][ppJ]-ppArray[ppI][0],ppArray[ppI][ppJ]-ppArray[ppI-1][ppJ]);
                #local ppNormalCount = ppNormalCount+1;
              #end
              #if (ppI<=ppIDim-2 & ppJ=ppJDim-1)
                #local ppNormal = ppNormal - vcross(ppArray[ppI][ppJ]-ppArray[ppI][0],ppArray[ppI][ppJ]-ppArray[ppI+1][ppJ]);
                #local ppNormalCount = ppNormalCount+1;    
              #end
              #if (ppI<=ppIDim-2 & ppJ=0)
                #local ppNormal = ppNormal + vcross(ppArray[ppI][ppJ]-ppArray[ppI][ppJDim-1],ppArray[ppI][ppJ]-ppArray[ppI+1][ppJ]);
                #local ppNormalCount = ppNormalCount+1;
              #end
              #if (ppI>=1 & ppJ=0)
                #local ppNormal = ppNormal - vcross(ppArray[ppI][ppJ]-ppArray[ppI][ppJDim-1],ppArray[ppI][ppJ]-ppArray[ppI-1][ppJ]);
                #local ppNormalCount = ppNormalCount+1;
              #end
            #end
            #if (ppI>=1 & ppJ<=ppJDim-2)
              #local ppNormal = ppNormal + vcross(ppArray[ppI][ppJ]-ppArray[ppI][ppJ+1],ppArray[ppI][ppJ]-ppArray[ppI-1][ppJ]);
              #local ppNormalCount = ppNormalCount+1;
            #end
            #if (ppI<=ppIDim-2 & ppJ<=ppJDim-2)
              #local ppNormal = ppNormal - vcross(ppArray[ppI][ppJ]-ppArray[ppI][ppJ+1],ppArray[ppI][ppJ]-ppArray[ppI+1][ppJ]);
              #local ppNormalCount = ppNormalCount+1;    
            #end
            #if (ppI<=ppIDim-2 & ppJ>=1 & ppJ<ppJDim)
              #local ppNormal = ppNormal + vcross(ppArray[ppI][ppJ]-ppArray[ppI][ppJ-1],ppArray[ppI][ppJ]-ppArray[ppI+1][ppJ]);
              #local ppNormalCount = ppNormalCount+1;
            #end
            #if (ppI>=1 & ppJ>=1 & ppJ<ppJDim)
              #local ppNormal = ppNormal - vcross(ppArray[ppI][ppJ]-ppArray[ppI][ppJ-1],ppArray[ppI][ppJ]-ppArray[ppI-1][ppJ]);
              #local ppNormalCount = ppNormalCount+1;
            #end
            #if (vlength(ppNormal/ppNormalCount)=0)
              <1,0,0>
            #else
              (ppNormal/ppNormalCount) 
            #end
          #end
          #local ppJ=ppJ+1;
        #end
        #local ppI=ppI+1;
      #end
    }
    uv_vectors{
      ppIDim*ppJTotal
      #local ppI=0;
      #while(ppI<ppIDim)
        #local ppJ=0;
        #while(ppJ<ppJTotal)
          <ppI/ppIDim,ppJ/ppJTotal>
          #local ppJ=ppJ+1;
          #end
        #local ppI=ppI+1;
      #end
    }
    face_indices{
      (ppIDim-1)*(ppJTotal-1)*2
      #local ppI=0;
      #while(ppI<ppIDim-1)
        #local ppJ=0;
        #while(ppJ<ppJTotal-1)
          <ppJ  + ppI   *ppJTotal, ppJ+(ppI+1)*ppJTotal, ppJ+1+ppI*ppJTotal>
          <ppJ+1+(ppI+1)*ppJTotal, ppJ+(ppI+1)*ppJTotal, ppJ+1+ppI*ppJTotal>
          #local ppJ=ppJ+1;
        #end
        #local ppI=ppI+1;
      #end
    }
    normal_indices{
      (ppIDim-1)*(ppJTotal-1)*2
      #local ppI=0;
      #while(ppI<ppIDim-1)
        #local ppJ=0;
        #while(ppJ<ppJTotal-1)
          <ppJ  + ppI   *ppJTotal, ppJ+(ppI+1)*ppJTotal, ppJ+1+ppI*ppJTotal>
          <ppJ+1+(ppI+1)*ppJTotal, ppJ+(ppI+1)*ppJTotal, ppJ+1+ppI*ppJTotal>
          #local ppJ=ppJ+1;
        #end
        #local ppI=ppI+1;
      #end
    }
    uv_indices{
      (ppIDim-1)*(ppJTotal-1)*2
      #local ppI=0;
      #while(ppI<ppIDim-1)
        #local ppJ=0;
        #while(ppJ<ppJTotal-1)
          <ppJ   +  ppI   *ppJTotal , ppJ + (ppI+1)*ppJTotal , ppJ+1 + ppI*ppJTotal>
          <ppJ+1 + (ppI+1)*ppJTotal , ppJ + (ppI+1)*ppJTotal , ppJ+1 + ppI*ppJTotal>
          #local ppJ=ppJ+1;
        #end
        #local ppI=ppI+1;
      #end
    }
  }
#end  

// The ppSeamFromArray macro takes a series of points defined in a 2D array, follows a line through those points and assigns 
// values to a second array. The first array would normally define a tubular form. The second array traces the length of the 
// tube defining the position of a seam. The values assigned to the second array add thickness and width to these positions to 
// define a strip intended to represent a simple seam in a piece of fabric.
// This is typically used in combination with the macros used to create items of clothing. Those macros 
// assign 2D arrays of points that are subsequently used to define a mesh of triangles.
//
// Parameters:
// -----------
//     ppSourceArray      - The array containing the points to be scanned
//     ppSourceNormal     - The array containing corresponding normals
//     ppIDim             - The size of the first dimension of the source array
//     ppJ                - The element to use from the second dimension of the source array
//     ppSeamArray        - The array into which the seam points will be written
//     ppSeamNormal       - The array into which corresponding normals will be written
//     ppK                - The cumulative index pointing to the second dimension of the seam array to be written
//     ppSeamThickness    - The thickness to add to the surface
//     ppSeamWidth        - The width of the band used to represent the seam

#macro ppSeamFromArray (ppSourceArray, ppSourceNormal, ppIDim, ppJ, ppSeamArray, ppSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
  #local ppI = 0;
  #while (ppI < ppIDim)
    #local ppSeamDisplacement   =  ppSeamWidth*vnormalize(ppSourceArray [ppI][ppJ+1]-ppSourceArray [ppI][ppJ])+ppSeamThickness*ppSourceNormal [ppI][ppJ+1];
    #local ppSeamArray [0][ppK] =  ppSourceArray [ppI][ppJ] + 0.2*ppSeamThickness*ppSourceNormal[ppI][ppJ];
    #local ppSeamNormal[0][ppK] =  ppSourceNormal[ppI][ppJ];
    #local ppSeamArray [1][ppK] =  ppSourceArray [ppI][ppJ] + ppSeamDisplacement;
    #local ppSeamNormal[1][ppK] =  ppSourceNormal[ppI][ppJ];
    #local ppSeamArray [2][ppK] =  ppSeamArray [1][ppK] - ppSeamThickness*ppSourceNormal[ppI][ppJ];
    #local ppSeamNormal[2][ppK] = -ppSourceNormal[ppI][ppJ];   
    #local ppI = ppI + 1;
    #local ppK = ppK + 1;  
  #end
  ppK; 
#end



// The ppDrawPoints macro takes a series of points defined in a 2D array and places a sphere at each point.
// This is typically used in combination with the macros used to create items of clothing. Those macros 
// assign 2D arrays of points that are subsequently used to define a mesh of triangles. This macro can therefore
// be used to 'illuminate' the points and is useful while debugging the generation of items of clothing.
//
// Parameters:
// -----------
//     ppArray            - The 2D array containing the set of points
//     ppIDim             - The size of the first dimension of the array
//     ppJDim             - The size of the second dimension of the array
//     ppRadius           - The radius of the sphere to be drawn at each point
//     ppColour           - The colour of the sphere to be drawn at each point

#macro ppDrawPoints(ppArray,ppIDim,ppJDim,ppRadius,ppColour)
  #local ppI = 0;
  #while (ppI<ppIDim)
    #local ppJ = 0;
    #while (ppJ<ppJDim)
//#if (ppJ = 4)
//      cylinder {ppArray[ppI][ppJ],ppArray[ppI][ppJ]+0.03*ppLeftThighNormal[ppI][ppJ],ppRadius/2 pigment {color rgb <2,2,2>}}
//#else
      #ifdef (ppArray[ppI][ppJ])
        sphere {ppArray[ppI][ppJ],ppRadius pigment {color rgb ppColour}}
      #end
//#end
      #local ppJ = ppJ + 1;
    #end
    #local ppI = ppI + 1;
  #end
#end
