/***************************************************************************************************/
/* ppshirts1.inc  - Copyright Chris Bartlett May 2005.                                             */
/* This work is licensed for royalty free re-use under the terms of the POV-Person License.        */
/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */
/***************************************************************************************************/
// To work out the line of the shirt we'll follow the anatomy up from the base of the abdomen into the neck.
// We'll take a series of slices through the anatomy and scan a series of segments on each slice. 
// The positions found will be adjusted to give the fabric depth and contours. 


#include "math.inc"
#include "ppclothingmacros.inc" 

#ifndef(ppVerbose)    #declare ppVerbose    = 0; #end
#ifndef(ppShowPoints) #declare ppShowPoints = 1; #end

// This macro generates a shirt by wrapping material around a series of objects representing elements of the torso.
//
// Parameters:
// -----------
//               - No parameters are passed to this macro. 
//
#macro ppShirtGenerator()
  //**************************************//
  // Default values for control variables //
  //**************************************//
  //#ifndef(ppShirtType)         #declare ppShirtType         = "Poloneck"; #end
  #ifndef(ppShirtWaistBandOn)  #declare ppShirtWaistBandOn  = 0;          #end   // Whether to create a waist band
  #ifndef(ppShouldersOn)       #declare ppShouldersOn       = 1;          #end   // Whether to cover the shoulders
  #ifndef(ppCollarOn)          #declare ppCollarOn          = 0;          #end   // Whether to create a simple collar
  #ifndef(ppShirtZipOn)        #declare ppShirtZipOn        = 0;          #end   // Whether to create a zip
  #ifndef(ppCollarWidth)       #declare ppCollarWidth       = 0.040;      #end     
  #ifndef(ppCollarThickness)   #declare ppCollarThickness   = 0.009;      #end     
  #ifndef(ppShirtWaistHeight)       #declare ppShirtWaistHeight       = 0.2;        #end   // How far above the hips
  #ifndef(ppShirtWaistBandHeight)   #declare ppShirtWaistBandHeight   = 0.025;      #end   // Height of the waist band
  #ifndef(ppZipFlapWidth)      #declare ppZipFlapWidth      = 0.020;      #end
  #ifndef(ppZipFlapThickness)  #declare ppZipFlapThickness  = 0.012;      #end
  
  // A spotted pigment is used to give a wrinkled texture.
  #ifndef(ppFabricDisplacementPigment) #declare ppFabricDisplacementPigment = pigment {spotted scale <0.2,0.02,0.2>*0.3} #end
  #ifndef(ppTopColour) #declare ppTopColour = <1,1,1>; #end
  #ifndef(ppTopTexture) 
    #declare ppTopTexture = texture {
      pigment {color rgb ppTopColour}  
  //    normal {bumps 0.8 scale <0.1,0.07,0.1>} 
  //    scale 0.5
    }
  #end
  #ifndef(ppShoulderTexture) #declare ppShoulderTexture = texture {ppTopTexture}  #end
  #ifndef(ppCollarTexture) 
    #declare ppCollarTexture = texture {
      pigment {color rgb ppTopColour}     
    }
  #end 
  #ifndef(ppShirtWaistBandTexture) 
    #declare ppShirtWaistBandTexture = texture {
      pigment {color rgb ppTopColour}           
    }
  #end 
  
  #ifndef(ppShirtFabricOffset)    #declare ppShirtFabricOffset = 0.015;     #end
  #ifndef(ppShirtFabricBumpiness) #declare ppShirtFabricBumpiness = 0.025;  #end
  
  
  // The number of slices determine the quality of each part 
  #ifndef(ppAbdomenSlices)   #declare ppAbdomenSlices    = 6;  #end
  #ifndef(ppChestSlices)     #declare ppChestSlices      = 14; #end
  #ifndef(ppShoulderSlices)  #declare ppShoulderSlices   = 6;  #end
  #ifndef(ppCollarSlices)    #declare ppCollarSlices     = 5;  #end
  #ifndef(ppShirtWaistBandSlices) #declare ppShirtWaistBandSlices  = 9;  #end  // Must be an odd number.
  
  // The number of segments should be consistent to enable different sections to be seamlessly stitched together
  #ifndef(ppShirtSegments)   #declare ppShirtSegments   = 20; #end
  
  
  //*******************//
  // Working variables //
  //*******************//
  #local ppSegments       = ppShirtSegments;
  #local ppZipSlices      = 3;    // Only alter if you alter the corresponding code
  
  // Local arrays to store surface positions and normals
  #local ppWaistBandArray  = array [ppWaistBandSlices][ppSegments];
  #local ppWaistBandNormal = array [ppWaistBandSlices][ppSegments];  
  #local ppAbdomenArray    = array [ppAbdomenSlices  ][ppSegments];
  #local ppAbdomenNormal   = array [ppAbdomenSlices  ][ppSegments];  
  #local ppChestArray      = array [ppChestSlices    ][ppSegments];
  #local ppChestNormal     = array [ppChestSlices    ][ppSegments];  
  #local ppShoulderArray   = array [ppShoulderSlices ][ppSegments];
  #local ppShoulderNormal  = array [ppShoulderSlices ][ppSegments];  
  #local ppCollarArray     = array [ppCollarSlices   ][ppSegments];
  #local ppCollarNormal    = array [ppCollarSlices   ][ppSegments];  
  #local ppZipArray        = array [ppZipSlices ][ppSegments/2];
  #local ppZipNormal       = array [ppZipSlices ][ppSegments/2];  
  
  
  // Local variables
  #local ppRotationDirection = 1;                                      
  #local ppRefLength         = vlength(ppAbdomenJointPos-ppNeckJointPos);              
  #local ppTraceLength       = 0.8*ppRefLength; 
  
  #local ppHipCentre           = (ppLeftHipPos+ppRightHipPos)/2;   // Mid point between the hips
  #local ppHipOrientation      = vnormalize(ppRightHipPos- ppLeftHipPos);
  #local ppShoulderOrientation = ppRightShoulderPos - ppLeftShoulderPos;
  #local ppBodyLine            = vaxis_rotate(vnormalize(ppAbdomenJointPos-ppHipCentre),ppHipOrientation,4);
  #if (ppShirtWaistBandOn)
    #local ppWaistBandStartPos = ppHipCentre + ppBodyLine*ppShirtWaistHeight;
    #local ppWaistBandEndPos   = ppHipCentre + ppBodyLine * (ppShirtWaistHeight+ppWaistBandHeight);
    #local ppAbdomenStartPos   = ppWaistBandEndPos;
  #else
    #local ppAbdomenStartPos   = ppHipCentre + ppBodyLine*ppShirtWaistHeight;
  #end
  #local ppAbdomenEndPos       = ppChestJointPos;
  #local ppChestStartPos       = ppAbdomenEndPos;
  #local ppChestEndPos         = ppNeckJointPos;
  #local ppShoulderStartPos    = ppChestEndPos;
  #local ppShoulderEndPos      = ppNeckJointPos + 0.99*(ppHeadJointPos-ppNeckJointPos);
  #local ppNeckStartPos        = ppShoulderEndPos;
  #local ppNeckEndPos          = ppHeadJointPos;
  #local ppAbdomenStartVector  = vnormalize(ppChestJointPos-ppAbdomenJointPos);
  #local ppAbdomenEndVector    = vaxis_rotate((vnormalize(ppAbdomenEndPos-ppAbdomenStartPos)+vnormalize(ppChestEndPos-ppChestStartPos))/2,ppShoulderOrientation,20);
  #local ppChestStartVector    = ppAbdomenEndVector;
  #local ppChestEndVector      = (vnormalize(ppChestEndPos-ppChestStartPos)+vnormalize(ppNeckEndPos-ppNeckStartPos))/2;
  #local ppShoulderStartVector = ppChestEndVector;
  #local ppShoulderEndVector   = vnormalize(ppNeckEndPos-ppNeckStartPos);
  #local ppNeckStartVector     = ppChestEndVector;
  #local ppNeckEndVector       = vnormalize(ppNeckEndPos-ppNeckStartPos);
  
  
  #local ppJoinEnds     = 1;
  #local ppDontJoinEnds = 0;
  //#local ppVerbose = 1; 
  
  #local ppEmptyArray = array[100][100]; 
  //#declare ppVerbose = 1;
  
  
  #if (ppVerbose = 1) #debug concat("Start of Abdomen part of Shirt","\n")  #end 
  #local ppStartRefVector = VPerp_To_Plane(ppHipOrientation,ppAbdomenStartVector);
  #local ppEndRefVector   = (ppStartRefVector + VPerp_To_Plane(ppShoulderOrientation,ppAbdomenEndVector))/2;
  ppLimbScan(ppTorsoObject,ppAbdomenStartPos,ppAbdomenStartVector,ppStartRefVector,ppAbdomenEndPos,ppAbdomenEndVector,ppEndRefVector,ppAbdomenSlices,ppSegments,ppAbdomenArray,ppAbdomenNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppAbdomenSlices-1, 0, ppSegments-1, ppAbdomenArray, ppAbdomenNormal, ppFabricDisplacementPigment,0,ppShirtFabricOffset)
  #declare ppAbdomen = ppMesh2FromArray(ppAbdomenArray,ppEmptyArray,ppAbdomenSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppAbdomenArray,ppAbdomenSlices,ppSegments,0.001,<1,2,1>) #end 
  
  
  #if (ppVerbose = 1) #debug concat("Start of Chest part of Shirt","\n")  #end 
  #local ppStartRefVector = ppEndRefVector;
  #local ppEndRefVector   = (ppStartRefVector + VPerp_To_Plane(ppShoulderOrientation,ppChestEndVector))/2;
  ppLimbScan(ppTorsoObject,ppChestStartPos,ppChestStartVector,ppStartRefVector,ppChestEndPos,ppChestEndVector,ppEndRefVector,ppChestSlices,ppSegments,ppChestArray,ppChestNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppChestSlices-1, 0, ppSegments-1, ppChestArray, ppChestNormal, ppFabricDisplacementPigment,0,ppShirtFabricOffset)
  ppStitchArrays(ppAbdomenArray,ppAbdomenNormal,ppAbdomenSlices-1,ppChestArray,ppChestNormal, 0, ppSegments)
  #declare ppChest = ppMesh2FromArray(ppChestArray,ppEmptyArray,ppChestSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppChestArray,ppChestSlices,ppSegments,0.001,<3,3,0>) #end 
  
  
  #if (ppShouldersOn)
    #if (ppVerbose = 1) #debug concat("Start of Shoulder part of Shirt","\n")  #end 
    #local ppStartRefVector = ppEndRefVector;
    ppStitchArrays(ppChestArray,ppChestNormal,ppChestSlices-1,ppShoulderArray,ppShoulderNormal, 0, ppSegments)
    ppEndScan(ppTorsoObject,ppChestEndVector,ppShoulderSlices,ppSegments,ppShoulderArray,ppShoulderNormal,ppRotationDirection,ppTraceLength,2)
    ppAddOffSetToArray(1, ppShoulderSlices-1, 0, ppSegments-1, ppShoulderArray, ppShoulderNormal, ppFabricDisplacementPigment,0,ppShirtFabricOffset)
    ppStretchPoints(ppTorsoObject,ppShoulderArray,0,ppShoulderSlices-1,0,ppSegments-1,ppShoulderEndVector,ppShoulderEndPos)
    #declare ppShoulder = ppMesh2FromArray(ppShoulderArray,ppEmptyArray,ppShoulderSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppShoulderArray,ppShoulderSlices,ppSegments,0.001,<2,0,0>) #end 
  #end
  
  
  //cylinder {ppShoulderEndPos,ppShoulderEndPos+ppShoulderOrientation*3 ,0.002 pigment {color rgb <2,0,2>}}
  
  
  
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXxxxx
  
  #if (ppShirtWaistBandOn) 
    #if (ppVerbose = 1) #debug concat("Start of Waist Band","\n")  #end 
    #local ppRefVector = -VPerp_To_Plane(ppHipOrientation,ppBodyLine);
    // The waist band is generated by scanning the hip object to get half the points. This half is copied in reverse order into the
    // second half, then the first half is offset outwards and the second half inwards.
    #local ppOuterSlices = (ppWaistBandSlices-1)/2;
    ppLimbScan(ppHipObject,ppWaistBandStartPos,ppBodyLine,ppRefVector,ppWaistBandEndPos,ppBodyLine,ppRefVector,ppOuterSlices,ppSegments,ppWaistBandArray,ppWaistBandNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppOuterSlices-1, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,ppShirtFabricOffset)
    ppAddOffSetToArray(0, 0, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,ppShirtFabricOffset*0.4)
    ppAddOffSetToArray(ppOuterSlices-1, ppOuterSlices-1, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,ppShirtFabricOffset*0.75)
    #local ppI = 0;
    #while (ppI<ppOuterSlices)
      #local ppJ = 0;
      #while (ppJ<ppSegments)
        #local ppWaistBandArray [ppWaistBandSlices-ppI-2][ppJ] = ppWaistBandArray [ppI][ppJ];
        #local ppWaistBandNormal[ppWaistBandSlices-ppI-2][ppJ] = ppWaistBandNormal[ppI][ppJ];
        #if (ppI = 0)
          #local ppWaistBandArray [ppWaistBandSlices-1][ppJ] = ppWaistBandArray [ppI][ppJ];
          #local ppWaistBandNormal[ppWaistBandSlices-1][ppJ] = ppWaistBandNormal[ppI][ppJ];
        #end
        #local ppJ = ppJ + 1;
      #end
      #local ppI = ppI + 1;
    #end
    ppAddOffSetToArray(ppOuterSlices, ppWaistBandSlices-2, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,-ppShirtFabricOffset*1.5)
    #declare ppWaistBand = ppMeshFromArray(ppWaistBandArray,ppWaistBandNormal,ppWaistBandSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppWaistBandArray,ppWaistBandSlices,ppSegments,0.001,<1,2,1>) #end 
  #end
  
  
  
  
  #if (ppShirtZipOn) 
  // Generate the Zip down the front of the shorts or trousers
    #if (ppVerbose = 1) #debug concat("Start of Zip at front of trousers","\n")  #end
    #local ppI = (ppCrotchSlices-1)/2;
    #local ppJ = 0;
    #while (ppJ < ppSegments/2)        // Front points only
      #local ppZipDisplacement   =  ppZipFlapWidth*vnormalize(ppCrotchArray [ppI+1][ppJ]-ppCrotchArray [ppI][ppJ]) + ppZipFlapThickness*VPerp_To_Plane(ppBodyLine, ppLeftHipPos-ppRightHipPos);  
      #local ppZipArray [0][ppJ] =  ppCrotchArray [ppI][ppJ];
      #local ppZipNormal[0][ppJ] =  ppCrotchNormal[ppI][ppJ];
      #local ppZipArray [1][ppJ] =  ppCrotchArray [ppI][ppJ] + ppZipDisplacement;
      #local ppZipNormal[1][ppJ] =  ppCrotchNormal[ppI][ppJ];
      #local ppZipArray [2][ppJ] =  ppZipArray [1][ppJ] - ppZipFlapThickness*ppCrotchNormal[ppI][ppJ];
      #local ppZipNormal[2][ppJ] = -ppCrotchNormal[ppI][ppJ];                                       
      #local ppJ = ppJ + 1;
    #end 
    #declare ppZip  = ppMeshFromArray(ppZipArray, ppZipNormal, ppZipSlices,ppSegments/2,ppDontJoinEnds)
  #end  
  // XXXXXXXXXXXXXXXXXXXXx end XXXXXXXXXXXXXXXXXXXXXXXXxx
  
  
  #declare ppShirt1 = union {
    object {ppAbdomen    texture {ppTopTexture}}  // rotate <ppXRot,0,ppZRot>}}  
    object {ppChest      texture {ppTopTexture}}  // rotate <ppXRot,0,ppZRot>}}  
    #if (ppShouldersOn)       object {ppShoulder   texture {ppShoulderTexture}}   #end
  //object {ppTorsoObject texture {ppSkinTexture1}}
    #if (ppCollarOn)          object {ppCollar     texture {ppCollarTexture}}     #end
    #if (ppShirtWaistBandOn)  object {ppWaistBand  texture {ppShirtWaistBandTexture}}  #end
    #if (ppShirtZipOn)        object {ppZip        texture {ppCollarTexture}}     #end
  }
  
  #declare ppShirt1 = difference {
    object {ppShirt1}
    sphere {<0,0,0>,0.2 pigment {color rgbt 1} scale <0.24,1,1> translate ppLeftShoulderPos +0.1*(ppLeftShoulderPos-ppRightShoulderPos)}
    sphere {<0,0,0>,0.2 pigment {color rgbt 1} scale <0.24,1,1> translate ppRightShoulderPos-0.1*(ppLeftShoulderPos-ppRightShoulderPos)}
    sphere {<0,0,0>,0.2 pigment {color rgbt 1} scale <0.5,1,1>  translate ppHeadJointPos}
  }           
  
    
  #declare ppShirtRef = ppAddBodyPart("Shirt",object{ppShirt1}); 

#end

ppTop()