/***************************************************************************************************/
/* ppshorts1.inc  - Copyright Chris Bartlett May 2005.                                             */
/* This work is licensed for royalty free re-use under the terms of the POV-Person License.        */
/* To view a copy of this license, visit http://www.telinco.co.uk/c_bartlett/license/license.htm   */
/***************************************************************************************************/
// To work out the line of the shorts we'll follow the anatomy up the left leg, through the Hip and down the right leg.
// We'll take a series of slices through the anatomy and scan a series of segments on each slice. 
// The positions found will be adjusted to give the fabric depth and contours.  


#include "math.inc"
#include "ppclothingmacros.inc" 

#ifndef(ppVerbose)    #declare ppVerbose    = 0; #end
#ifndef(ppShowPoints) #declare ppShowPoints = 1; #end

// This macro generates a pair of trousers or shorts by wrapping material around a series of objects representing elements of 
// the legs and lower body.
//
// Parameters:
// -----------
//               - No parameters are passed to this macro. 
//
#macro ppShortsGenerator ()
  //**************************************//
  // Default values for control variables //
  //**************************************//
  #ifndef(ppTrousersOrShorts)  #declare ppTrousersOrShorts  = "Trousers"; #end   // Trousers include lower leg. Shorts don't.
  //#ifndef(ppLegLength)         #declare ppLegLength         = 0.9;        #end   // How far down the leg section (0-1)
  #ifndef(ppWaistBandOn)       #declare ppWaistBandOn       = 1;          #end   // Whether to create a waist band
  #ifndef(ppBeltHoopsOn)       #declare ppBeltHoopsOn       = 1;          #end   // Whether to create belt hoops around the waist band
  #ifndef(ppEndSeamOn)         #declare ppEndSeamOn         = 1;          #end   // Whether to create a seam at the leg ends
  #ifndef(ppSideSeamOn)        #declare ppSideSeamOn        = 1;          #end   // Whether to create a seam down the sides
  #ifndef(ppInsideSeamOn)      #declare ppInsideSeamOn      = 1;          #end   // Whether to create a seam down the inside of the legs
  #ifndef(ppBackSeamOn)        #declare ppBackSeamOn        = 1;          #end   // Whether to create a seam down the Back
  #ifndef(ppZipOn)             #declare ppZipOn             = 1;          #end   // Whether to create a zip
  #ifndef(ppSeamWidth)         #declare ppSeamWidth         = 0.012;      #end     
  #ifndef(ppSeamThickness)     #declare ppSeamThickness     = 0.003;      #end     
  #ifndef(ppWaistHeight)       #declare ppWaistHeight       = 0.20;       #end   // How far above the hips
  #ifndef(ppWaistBandHeight)   #declare ppWaistBandHeight   = 0.025;      #end   // Height of the waist band
  #ifndef(ppBeltHoopHeight)    #declare ppBeltHoopHeight    = 0.03;       #end   // Height of the belt hoops
  #ifndef(ppBeltHoopWidth)     #declare ppBeltHoopWidth     = 0.012;      #end    
  #ifndef(ppBeltHoopThickness) #declare ppBeltHoopThickness = 0.003;      #end    
  #ifndef(ppBeltHoopQuantity)  #declare ppBeltHoopQuantity  = 6;          #end   // The number of hoops to hold a belt on the waist band 
  #ifndef(ppZipFlapWidth)      #declare ppZipFlapWidth      = 0.025;      #end
  #ifndef(ppZipFlapThickness)  #declare ppZipFlapThickness  = 0.012;      #end
  
  // A spotted pigment is used to give a wrinkled texture.
  #ifndef(ppFabricDisplacementPigment) #declare ppFabricDisplacementPigment = pigment {spotted scale <0.2,0.02,0.2>*0.3} #end
  #ifndef(ppTrouserLegTexture) 
    #declare ppTrouserLegTexture = texture {
      pigment {color rgb <0.4,0.6,1>}  
      normal {bumps 0.8 scale <0.1,0.07,0.1>} 
      scale 0.5
    }
  #end
  #ifndef(ppTrouserWaistBandTexture) 
    #declare ppTrouserWaistBandTexture = texture {
      pigment {color rgb <0.4,0.6,1>}            //  <0.3,0.2,0.1>
    }
  #end 
  #ifndef(ppTrouserSeamTexture) 
    #declare ppTrouserSeamTexture = texture {
      pigment {color rgb <0.4,0.6,1>}            //  <0.3,0.2,0.1>
    }
  #end 
  
  #ifndef(ppFabricOffSet)    #declare ppFabricOffSet = 0.005;     #end
  #ifndef(ppFabricBumpiness) #declare ppFabricBumpiness = 0.025;  #end
  
  
  // The number of slices determine the quality of each part 
  #ifndef(ppLeftThighSlices)  #declare ppLeftThighSlices  = 12;                #end
  #ifndef(ppRightThighSlices) #declare ppRightThighSlices = ppLeftThighSlices; #end
  #ifndef(ppKneeSlices)       #declare ppKneeSlices       = 5;                 #end
  #ifndef(ppLeftCalfSlices)   #declare ppLeftCalfSlices   = 12;                #end
  #ifndef(ppRightCalfSlices)  #declare ppRightCalfSlices  = ppLeftCalfSlices;  #end
  #ifndef(ppLeftHipSlices)    #declare ppLeftHipSlices    = 8;                 #end
  #ifndef(ppRightHipSlices)   #declare ppRightHipSlices   = 8;                 #end
  #ifndef(ppCrotchSlices)     #declare ppCrotchSlices     = 11;                #end  // Must be an odd number.
  #ifndef(ppWaistBandSlices)  #declare ppWaistBandSlices  = 9;                 #end  // Must be an odd number.
  
  // The number of segments should be consistent to enable different sections to be seamlessly stitched together
  #ifndef(ppShortsSegments)   #declare ppShortsSegments   = 20;                #end
  
  
  //*******************//
  // Working variables //
  //*******************//
  #local ppSegments       = ppShortsSegments;
  #local ppSideSeamSlices = (ppLeftCalfSlices+ppKneeSlices*2+ppLeftThighSlices+ppLeftHipSlices+ppCrotchSlices
                             +ppRightHipSlices+ppRightThighSlices+ppKneeSlices*2+ppRightCalfSlices);
  #local ppSeamSlices     = 3;    // Only alter if you alter the corresponding code
  #local ppZipSlices      = 3;    // Only alter if you alter the corresponding code
  
  // Local arrays to store surface positions and normals
  #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers
    #local ppLeftLowerKneeArray   = array [ppKneeSlices     ][ppSegments];
    #local ppLeftLowerKneeNormal  = array [ppKneeSlices     ][ppSegments];  
    #local ppLeftUpperKneeArray   = array [ppKneeSlices     ][ppSegments];
    #local ppLeftUpperKneeNormal  = array [ppKneeSlices     ][ppSegments];  
    #local ppLeftCalfArray        = array [ppLeftCalfSlices ][ppSegments];
    #local ppLeftCalfNormal       = array [ppLeftCalfSlices ][ppSegments];  
    #local ppRightLowerKneeArray  = array [ppKneeSlices     ][ppSegments];
    #local ppRightLowerKneeNormal = array [ppKneeSlices     ][ppSegments];  
    #local ppRightUpperKneeArray  = array [ppKneeSlices     ][ppSegments];
    #local ppRightUpperKneeNormal = array [ppKneeSlices     ][ppSegments];  
    #local ppRightCalfArray       = array [ppRightCalfSlices][ppSegments];
    #local ppRightCalfNormal      = array [ppRightCalfSlices][ppSegments];  
  #end
  #local ppLeftThighArray       = array [ppLeftThighSlices ][ppSegments];
  #local ppLeftThighNormal      = array [ppLeftThighSlices ][ppSegments];  
  #local ppLeftHipArray         = array [ppLeftHipSlices   ][ppSegments];
  #local ppLeftHipNormal        = array [ppLeftHipSlices   ][ppSegments];  
  #local ppLeftEndSeamArray     = array [ppSeamSlices      ][ppSegments];
  #local ppLeftEndSeamNormal    = array [ppSeamSlices      ][ppSegments];  
  #local ppRightThighArray      = array [ppRightThighSlices][ppSegments];
  #local ppRightThighNormal     = array [ppRightThighSlices][ppSegments];  
  #local ppRightHipArray        = array [ppRightHipSlices  ][ppSegments];
  #local ppRightHipNormal       = array [ppRightHipSlices  ][ppSegments];  
  #local ppRightEndSeamArray    = array [ppSeamSlices      ][ppSegments];
  #local ppRightEndSeamNormal   = array [ppSeamSlices      ][ppSegments];  
  #local ppCrotchArray          = array [ppCrotchSlices    ][ppSegments];
  #local ppCrotchNormal         = array [ppCrotchSlices    ][ppSegments];  
  #local ppWaistBandArray       = array [ppWaistBandSlices ][ppSegments];
  #local ppWaistBandNormal      = array [ppWaistBandSlices ][ppSegments];  
  #local ppBeltHoopArray        = array [6][4];
  #local ppBeltHoopNormal       = array [6][4];  
  #local ppSideSeamArray        = array [ppSeamSlices][ppSideSeamSlices];
  #local ppSideSeamNormal       = array [ppSeamSlices][ppSideSeamSlices];  
  #local ppInsideSeamArray      = array [ppSeamSlices][ppSideSeamSlices];
  #local ppInsideSeamNormal     = array [ppSeamSlices][ppSideSeamSlices];  
  #local ppBackSeamArray        = array [ppSeamSlices][ppSegments/2];
  #local ppBackSeamNormal       = array [ppSeamSlices][ppSegments/2];  
  #local ppZipArray             = array [ppZipSlices ][ppSegments/2];
  #local ppZipNormal            = array [ppZipSlices ][ppSegments/2];  
  
  
  // Local variables
  #local ppRotationDirection = 1;
  #local ppRefLength = vlength(ppLeftHipPos-ppLeftKneePos);
  #local ppTraceLength = 0.8*ppRefLength;
  
  #local ppHipCentre         = (ppLeftHipPos+ppRightHipPos)/2;   // Mid point between the hips
  #local ppBodyLine          = vaxis_rotate(vnormalize(ppBodyCentre-ppHipCentre),ppRightHipPos-ppLeftHipPos,14);
  #local ppWaistBandStartPos = ppHipCentre + ppBodyLine *(ppWaistHeight-ppWaistBandHeight);
  #local ppWaistBandEndPos   = ppHipCentre + ppBodyLine * ppWaistHeight;
  #local ppCrotchTopPos      = ppWaistBandStartPos + ppBodyLine*0.25*vlength(ppWaistBandStartPos-ppWaistBandEndPos);
  
  // Work out the distance from the centre point between the two hip joints to the front surface of the abdomen
  #local ppNorm = <0,0,0>;
  #local ppRayDirection = VPerp_To_Plane(ppBodyLine, ppLeftHipPos-ppRightHipPos);
  #local ppThisPoint = trace(ppHipObject, ppHipCentre-ppRayDirection, ppRayDirection, ppNorm);
  #if (vlength(ppNorm)!=0)
    #local ppMinOffset    =  vlength(ppHipCentre - ppThisPoint);
  #else
    #local ppMinOffset    = 0;
    #warning concat("The ray from the centre of the body <",vstr(3,ppHipCentre,",",0,3),"> didn't intercept the hip object.\n")
  #end
  
  // Calculate Left Leg positions and vectors
  #if (strcmp(ppTrousersOrShorts,"Shorts")=0)         // If we're doing Shorts
    #if (ppEndSeamOn) #local ppLeftThighStartPos = ppLeftHipPos  - (ppLeftHipPos-ppLeftKneePos)* ppLegLength;
    #else             #local ppLeftThighStartPos = ppLeftHipPos  - (ppLeftHipPos-ppLeftKneePos)*(ppLegLength-ppSeamWidth); 
    #end
  #else                                               // If we're doing Trousers
    #local ppLeftThighStartPos = ppLeftHipPos  - (ppLeftHipPos -ppLeftKneePos)*0.9;
    #if (ppEndSeamOn) #local ppLeftCalfStartPos  = ppLeftKneePos - (ppLeftKneePos-ppLeftAnklePos)*ppLegLength;
    #else             #local ppLeftCalfStartPos  = ppLeftKneePos - (ppLeftKneePos-ppLeftAnklePos)*(ppLegLength-ppSeamWidth);
    #end
    #local ppLeftCalfEndPos      = ppLeftKneePos - (ppLeftKneePos-ppLeftAnklePos)*0.1;
    #local ppLeftCalfStartVector = vnormalize(ppLeftKneePos-ppLeftAnklePos);
  #end
  #local ppLeftCalfEndVector   = vnormalize(ppLeftKneePos-ppLeftAnklePos);
  #local ppLeftThighStartVector = vnormalize(ppLeftHipPos-ppLeftKneePos); 
  #local ppNorm = <0,0,0>;
  #local ppRayDirection = ppLeftHipPos-ppLeftKneePos;
  #local ppThisPoint = trace(ppHipObject, ppLeftHipPos-ppRayDirection, ppRayDirection, ppNorm);
  #if (vlength(ppNorm)!=0)
    #if (vlength(ppLeftHipPos-ppThisPoint)<ppMinOffset)  
      #local ppLeftThighEndPos    =  ppLeftHipPos - ppMinOffset*vnormalize(ppRayDirection);
    #else 
      #local ppLeftThighEndPos    =  ppThisPoint; 
    #end
  #else
    #local ppLeftThighEndPos    = 0;
    #warning concat("The ray from the left hip <",vstr(3,ppLeftHipPos,",",0,3),"> didn't intercept the hip object.\n")
  #end
  #local ppLeftThighEndVector = vnormalize(vnormalize(ppLeftHipPos-ppLeftKneePos)+0.6*ppBodyLine+0.5*vnormalize(ppRightHipPos-ppLeftHipPos));
  #local ppLeftKneeVector = vnormalize(ppLeftCalfEndVector+ppLeftThighStartVector);
  
  
  // Calculate Left Hip positions and vectors
  #local ppLeftHipStartPos    = ppLeftThighEndPos;
  #local ppLeftHipStartVector = ppLeftThighEndVector;
  #local ppLeftHipEndPos      = ppLeftHipPos + 0.25*(ppRightHipPos-ppLeftHipPos);
  #local ppLeftHipEndVector   = vnormalize(0.3*vnormalize(ppLeftHipPos-ppLeftKneePos)+0.6*ppBodyLine+0.9*vnormalize(ppRightHipPos-ppLeftHipPos));
  
  
  
  // Calculate Right Leg positions and vectors
  #if (strcmp(ppTrousersOrShorts,"Shorts")=0)         // If we're doing Shorts
    #if (ppEndSeamOn) #local ppRightThighEndPos = ppRightHipPos  - (ppRightHipPos-ppRightKneePos)* ppLegLength;
    #else             #local ppRightThighEndPos = ppRightHipPos  - (ppRightHipPos-ppRightKneePos)*(ppLegLength-ppSeamWidth); 
    #end
  #else                                               // If we're doing Trousers
    #local ppRightThighEndPos = ppRightHipPos  - (ppRightHipPos -ppRightKneePos)*0.9;
    #if (ppEndSeamOn) #local ppRightCalfEndPos  = ppRightKneePos - (ppRightKneePos-ppRightAnklePos)*ppLegLength;
    #else             #local ppRightCalfEndPos  = ppRightKneePos - (ppRightKneePos-ppRightAnklePos)*(ppLegLength-ppSeamWidth);
    #end
    #local ppRightCalfStartPos     = ppRightKneePos - (ppRightKneePos-ppRightAnklePos)*0.1;
    #local ppRightCalfStartVector  = -vnormalize(ppRightKneePos-ppRightAnklePos);
  #end
  #local ppRightCalfEndVector    = -vnormalize(ppRightKneePos-ppRightAnklePos);
  #local ppRightThighEndVector = -vnormalize(ppRightHipPos-ppRightKneePos); 
  #local ppNorm = <0,0,0>;
  #local ppRayDirection = ppRightHipPos-ppRightKneePos;
  #local ppThisPoint = trace(ppHipObject, ppRightHipPos-ppRayDirection, ppRayDirection, ppNorm);
  #if (vlength(ppNorm)!=0)
    #if (vlength(ppRightHipPos-ppThisPoint)<ppMinOffset)  
      #local ppRightThighStartPos    =  ppRightHipPos - ppMinOffset*vnormalize(ppRayDirection);
    #else 
      #local ppRightThighStartPos    =  ppThisPoint; 
    #end
  #else
    #local ppRightThighStartPos    = 0;
    #warning concat("The ray from the Right hip <",vstr(3,ppRightHipPos,",",0,3),"> didn't intercept the hip object.\n")
  #end
  #local ppRightThighStartVector = -vnormalize(vnormalize(ppRightHipPos-ppRightKneePos)+0.6*ppBodyLine+0.5*vnormalize(ppLeftHipPos-ppRightHipPos));
  #local ppRightKneeVector = vnormalize(ppRightCalfEndVector+ppRightThighStartVector);
  
  // Calculate Right Hip positions and vectors
  #local ppRightHipEndPos      = ppRightThighStartPos;
  #local ppRightHipEndVector   = ppRightThighStartVector;
  #local ppRightHipStartPos    = ppRightHipPos + 0.25*(ppLeftHipPos-ppRightHipPos);
  #local ppRightHipStartVector = -vnormalize(0.3*vnormalize(ppRightHipPos-ppRightKneePos)+0.6*ppBodyLine+0.9*vnormalize(ppLeftHipPos-ppRightHipPos));
  
  
  
  
  // Create various sections of the objects to be scanned.
  // These sections reduce the risk of incorrect scans due to overlaps etc.
  #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers
    #local ppLeftLowerKneeSection = difference {
      merge {
        object {ppLeftCalfObject}
        object {ppLeftThighObject}
      }
      plane { ppLeftCalfEndVector,0 translate ppLeftCalfEndPos-0.005*vnormalize(ppLeftKneePos-ppLeftAnklePos)}
      plane {-ppLeftKneeVector   ,0 translate ppLeftKneePos   +0.005*vnormalize(ppLeftKneePos-ppLeftAnklePos)}
    }
    #local ppLeftUpperKneeSection = difference {
      merge {
        object {ppLeftCalfObject}
        object {ppLeftThighObject}
      }
      plane {-ppLeftThighStartVector,0 translate ppLeftThighStartPos+0.005*vnormalize(ppLeftHipPos-ppLeftKneePos)}
      plane { ppLeftKneeVector      ,0 translate ppLeftKneePos      -0.005*vnormalize(ppLeftKneePos-ppLeftAnklePos)}
    }
    #local ppRightLowerKneeSection = difference {
      merge {
        object {ppRightCalfObject}
        object {ppRightThighObject}
      }
      plane {-ppRightCalfStartVector,0 translate ppRightCalfStartPos-0.005*vnormalize(ppRightKneePos-ppRightAnklePos)}
      plane { ppRightKneeVector     ,0 translate ppRightKneePos     +0.005*vnormalize(ppRightKneePos-ppRightAnklePos)}
    }
    #local ppRightUpperKneeSection = difference {
      merge {
        object {ppRightCalfObject}
        object {ppRightThighObject}
      }
      plane { ppRightThighEndVector,0 translate ppRightThighEndPos+0.005*vnormalize(ppRightHipPos-ppRightKneePos)}
      plane {-ppRightKneeVector    ,0 translate ppRightKneePos    -0.005*vnormalize(ppRightKneePos-ppRightAnklePos)}
    }
  #end
  #local ppLeftHipSection = merge {
    difference { 
      object {ppLeftThighObject}
      plane {ppLeftThighEndVector,0 translate ppLeftThighEndPos-0.005*vnormalize(ppLeftHipPos-ppLeftKneePos)}
    }
    difference { 
      object {ppHipObject}
      plane {-ppLeftHipEndVector,0 translate ppLeftHipEndPos+0.005*ppLeftHipEndVector}
      plane {-ppBodyLine,0 translate ppWaistBandStartPos}
    }
  }
  
  #local ppRightHipSection = merge {
    difference { 
      object {ppRightThighObject}
      plane {-ppRightThighStartVector,0 translate ppRightThighStartPos-0.005*vnormalize(ppRightHipPos-ppRightKneePos)}
    }
    difference { 
      object {ppHipObject}
      plane { ppRightHipStartVector,0 translate ppRightHipStartPos-0.005*ppRightHipStartVector}
      plane {-ppBodyLine,0 translate ppWaistBandStartPos}
    }
  }
  
  #local ppCrotchSection = difference {
    merge {
      difference { 
        object {ppLeftThighObject}
        plane {ppLeftThighEndVector,0 translate ppLeftThighEndPos-0.005*vnormalize(ppLeftHipPos-ppLeftKneePos)}
      }
      difference { 
        object {ppRightThighObject}
        plane {-ppRightThighStartVector,0 translate ppRightThighStartPos-0.005*vnormalize(ppRightHipPos-ppRightKneePos)}
      }
      difference { 
        object {ppHipObject}
        plane {-ppBodyLine,0 translate ppCrotchTopPos}
      }
    }
    plane { ppLeftHipEndVector   ,0 translate ppLeftHipEndPos   -0.005*ppLeftHipEndVector}
    plane {-ppRightHipStartVector,0 translate ppRightHipStartPos+0.005*ppRightHipStartVector}
  }
  
  
  // ppLeftHipPos     
  // ppRightHipPos    
  // ppLeftKneePos    
  // ppRightKneePos   
  // ppLeftAnklePos   
  // ppRightAnklePos  
  // ppBodyCentre     
  //                   
  // ppLeftThighObject 
  // ppRightThighObject
  // ppLeftCalfObject  
  // ppRightCalfObject 
  // ppHipObject       
  
  
  
  
  //#local ppLeftLegJointVector = -vnormalize(0.55*vnormalize(ppLeftLegEndPos-ppLeftLegJointPos)   + 0.55*vnormalize(ppLeftLegJointPos-ppRightLegJointPos) - 0.35*ppBodyLine);
  //#local ppRightLegJointVector = vnormalize(0.55*vnormalize(ppRightLegEndPos-ppRightLegJointPos) + 0.55*vnormalize(ppRightLegJointPos-ppLeftLegJointPos) - 0.35*ppBodyLine);
    
  
  
  //
  //
  //// We need to calculate the angles required to orient the bumpy texture to the line of the leg
  ////#local ppLegVector = ppCoreEndPos - ppCoreStartPos;
  ////#local ppLegTempVector = VProject_Plane(ppLegVector, z);
  ////#local ppXRot = VAngleD(ppLegTempVector,ppLegVector);
  ////#if (ppLegVector.z<0) #local ppXRot = -ppXRot; #end
  ////#local ppZRot = VAngleD(y,ppLegTempVector);
  ////#if (ppLegVector.x<0) #local ppZRot = -ppZRot; #end
  //
  //#local ppHipSection = intersection {
  //  object {ppHipObject}
  //  plane {ppBodyLine,0 translate ppWaistPosition pigment {color rgb <0,1,1>}}
  //}
  //
  
  
  
  #local ppJoinEnds     = 1;
  #local ppDontJoinEnds = 0;
  //#local ppVerbose = 1;
  
  #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers 
    #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Lower Left Leg","\n")  #end 
    #local ppStartRefVector = VPerp_To_Plane(ppLeftKneePos-ppLeftHipPos,ppLeftAnklePos-ppLeftKneePos);  // Doesn't work with straight leg.
    #local ppEndRefVector   = ppStartRefVector;
    ppLimbScan(ppLeftCalfObject,ppLeftCalfStartPos,ppLeftCalfStartVector,ppStartRefVector,ppLeftCalfEndPos,ppLeftCalfEndVector,ppEndRefVector,ppLeftCalfSlices,ppSegments,ppLeftCalfArray,ppLeftCalfNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppLeftCalfSlices-1, 0, ppSegments-1, ppLeftCalfArray, ppLeftCalfNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
    #local ppLeftCalf = ppMeshFromArray(ppLeftCalfArray,ppLeftCalfNormal,ppLeftCalfSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppLeftCalfArray,ppLeftCalfSlices,ppSegments,0.001,<0,1,0>) #end   
  
    #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Left Lower Knee Section","\n")  #end 
    #local ppStartRefVector = ppEndRefVector;
    #local ppEndRefVector   = ppStartRefVector;
    ppLimbScan(ppLeftLowerKneeSection,ppLeftCalfEndPos,ppLeftCalfEndVector,ppStartRefVector,ppLeftKneePos,ppLeftKneeVector,ppEndRefVector,ppKneeSlices,ppSegments,ppLeftLowerKneeArray,ppLeftLowerKneeNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppKneeSlices-1, 0, ppSegments-1, ppLeftLowerKneeArray, ppLeftLowerKneeNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
    #local ppLeftLowerKnee = ppMeshFromArray(ppLeftLowerKneeArray,ppLeftLowerKneeNormal,ppKneeSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppLeftLowerKneeArray,ppKneeSlices,ppSegments,0.001,<0,1,0>) #end   
    
    #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Left Upper Knee Section","\n")  #end 
    #local ppStartRefVector = ppEndRefVector;
    #local ppEndRefVector   = ppStartRefVector;
    ppLimbScan(ppLeftUpperKneeSection,ppLeftKneePos,ppLeftKneeVector,ppStartRefVector,ppLeftThighStartPos,ppLeftThighStartVector,ppEndRefVector,ppKneeSlices,ppSegments,ppLeftUpperKneeArray,ppLeftUpperKneeNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppKneeSlices-1, 0, ppSegments-1, ppLeftUpperKneeArray, ppLeftUpperKneeNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
    #local ppLeftUpperKnee = ppMeshFromArray(ppLeftUpperKneeArray,ppLeftUpperKneeNormal,ppKneeSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppLeftUpperKneeArray,ppKneeSlices,ppSegments,0.001,<0,1,0>) #end   
  #end
  
  //#declare ppShowPoints = 1;
  
  // Generate the Upper Left Leg Section of the Shorts or Trousers
  // The reference vector is used to consistently orient the starting point on each circular scan.
  // It contributes to the ease with which various generated pieces can be seamlessly assembled.
  // We point it in the direction of the inside seam of the shorts/trousers. 
  #local ppPerpRefVector  = -VPerp_To_Plane(ppLeftKneePos-ppLeftHipPos, ppRightHipPos-ppLeftHipPos); // A line out from Left Buttock
  #local ppStartRefVector =  VPerp_To_Plane(ppLeftKneePos-ppLeftHipPos,ppPerpRefVector);
  #local ppEndRefVector   = vaxis_rotate(ppStartRefVector,ppPerpRefVector,45);
  #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Upper Left Leg","\n")  #end 
  ppLimbScan(ppLeftThighObject,ppLeftThighStartPos,ppLeftThighStartVector,ppStartRefVector,ppLeftThighEndPos,ppLeftThighEndVector,ppEndRefVector,ppLeftThighSlices,ppSegments,ppLeftThighArray,ppLeftThighNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppLeftThighSlices-1, 0, ppSegments-1, ppLeftThighArray, ppLeftThighNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
  //ppStretchPoints(ppHipSection,ppLeftLegArray,0,ppLeftLegSlices-1,0,ppSegments-1,ppBodyLine,ppWaistPosition)
  #local ppLeftThigh = ppMeshFromArray(ppLeftThighArray,ppLeftThighNormal,ppLeftThighSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppLeftThighArray,ppLeftThighSlices,ppSegments,0.001,<0,1,0>) #end   
  
  #declare ppShowPoints = 0;
  
  
  // Generate the Left Hip Section of the Shorts or Trousers
  #local ppStartRefVector = ppEndRefVector;
  #local ppEndRefVector   = vnormalize(-ppBodyLine+0.2*vnormalize(ppLeftKneePos-ppLeftHipPos));
  #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Left Hip section","\n")  #end 
  ppLimbScan(ppLeftHipSection,ppLeftHipStartPos,ppLeftHipStartVector,ppStartRefVector,ppLeftHipEndPos,ppLeftHipEndVector,ppEndRefVector,ppLeftHipSlices,ppSegments,ppLeftHipArray,ppLeftHipNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppLeftHipSlices-1, 0, ppSegments-1, ppLeftHipArray, ppLeftHipNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
  #local ppLeftHip = ppMeshFromArray(ppLeftHipArray,ppLeftHipNormal,ppLeftHipSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppLeftHipArray,ppLeftHipSlices,ppSegments,0.001,<0,1,0>) #end 
  
  //#declare ppShowPoints=1;
  //#declare ppVerbose   =1;
  
  // Generate the Crotch Section of the Shorts or Trousers 
  #local ppStartRefVector = ppEndRefVector;
  #local ppEndRefVector   = vnormalize(-ppBodyLine+0.2*vnormalize(ppRightKneePos-ppRightHipPos));
  #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Crotch section","\n")  #end 
  ppLimbScan(ppCrotchSection,ppLeftHipEndPos,ppLeftHipEndVector,ppStartRefVector,ppRightHipStartPos,ppRightHipStartVector,ppEndRefVector,ppCrotchSlices,ppSegments,ppCrotchArray,ppCrotchNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppCrotchSlices-1, 0, ppSegments-1, ppCrotchArray, ppCrotchNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
  ppStretchPoints(ppLeftThighObject,ppCrotchArray,0,ppCrotchSlices-1,0,ppSegments-1,-ppLeftThighEndVector   ,ppLeftThighEndPos   )
  ppStretchPoints(ppRightThighObject,ppCrotchArray,0,ppCrotchSlices-1,0,ppSegments-1, ppRightThighStartVector,ppRightThighStartPos)
  ppStretchPoints(ppCrotchSection,ppCrotchArray,0,ppCrotchSlices-1,0,ppSegments-1,ppBodyLine,ppCrotchTopPos)
  #local ppCrotch = ppMeshFromArray(ppCrotchArray,ppCrotchNormal,ppCrotchSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppCrotchArray,ppCrotchSlices,ppSegments,0.001,<2,0,0>) #end 
  
  #declare ppShowPoints=0;
  //#declare ppVerbose   =0;
  
  // Generate the Right Hip Section of the Shorts or Trousers
  #local ppStartRefVector = ppEndRefVector;
  #local ppPerpRefVector  =  VPerp_To_Plane(ppRightKneePos-ppRightHipPos, ppLeftHipPos-ppRightHipPos); // A line out from Right Buttock
  #local ppTempRefVector  = -VPerp_To_Plane(ppRightKneePos-ppRightHipPos,ppPerpRefVector);
  #local ppEndRefVector   =  vaxis_rotate(ppTempRefVector,ppPerpRefVector,-45);
  #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Right Hip section","\n")  #end 
  ppLimbScan(ppRightHipSection,ppRightHipStartPos,ppRightHipStartVector,ppStartRefVector,ppRightHipEndPos,ppRightHipEndVector,ppEndRefVector,ppRightHipSlices,ppSegments,ppRightHipArray,ppRightHipNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppRightHipSlices-1, 0, ppSegments-1, ppRightHipArray, ppRightHipNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
  #local ppRightHip = ppMeshFromArray(ppRightHipArray,ppRightHipNormal,ppRightHipSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppRightHipArray,ppRightHipSlices,ppSegments,0.001,<0,1,0>) #end 
  
  
  // Generate the Upper Right Leg Section of the Shorts or Trousers
  #local ppStartRefVector = ppEndRefVector;
  //#local ppPerpRefVector  = -VPerp_To_Plane(ppRightKneePos-ppRightHipPos, ppLeftHipPos-ppRightHipPos);
  #local ppEndRefVector   = -VPerp_To_Plane(ppRightKneePos-ppRightHipPos,ppPerpRefVector);
  //#local ppStartRefVector =  vaxis_rotate(ppEndRefVector,ppPerpRefVector,-45);
  #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Upper Right Leg","\n")  #end 
  ppLimbScan(ppRightThighObject,ppRightThighStartPos,ppRightThighStartVector,ppStartRefVector,ppRightThighEndPos,ppRightThighEndVector,ppEndRefVector,ppRightThighSlices,ppSegments,ppRightThighArray,ppRightThighNormal,0,ppRotationDirection,ppTraceLength)
  ppAddOffSetToArray(0, ppRightThighSlices-1, 0, ppSegments-1, ppRightThighArray, ppRightThighNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
  //ppStretchPoints(ppHipSection,ppRightLegArray,0,ppRightLegSlices-1,0,ppSegments-1,ppBodyLine,ppWaistPosition)
  #local ppRightThigh = ppMeshFromArray(ppRightThighArray,ppRightThighNormal,ppRightThighSlices,ppSegments,ppJoinEnds)
  #if (ppShowPoints=1) ppDrawPoints(ppRightThighArray,ppRightThighSlices,ppSegments,0.001,<0,1,0>) #end 
  
  #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers 
    #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Right Upper Knee Section","\n")  #end 
    #local ppStartRefVector = ppEndRefVector;
    #local ppEndRefVector   = ppStartRefVector;
    ppLimbScan(ppRightUpperKneeSection,ppRightThighEndPos,ppRightThighEndVector,ppStartRefVector,ppRightKneePos,ppRightKneeVector,ppEndRefVector,ppKneeSlices,ppSegments,ppRightUpperKneeArray,ppRightUpperKneeNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppKneeSlices-1, 0, ppSegments-1, ppRightUpperKneeArray, ppRightUpperKneeNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
    #local ppRightUpperKnee = ppMeshFromArray(ppRightUpperKneeArray,ppRightUpperKneeNormal,ppKneeSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppRightUpperKneeArray,ppKneeSlices,ppSegments,0.001,<0,1,0>) #end   
  
    #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Left Lower Knee Section","\n")  #end 
    #local ppStartRefVector = ppEndRefVector;
    #local ppEndRefVector   = ppStartRefVector;
    ppLimbScan(ppRightLowerKneeSection,ppRightKneePos,ppRightKneeVector,ppStartRefVector,ppRightCalfStartPos,ppRightCalfStartVector,ppEndRefVector,ppKneeSlices,ppSegments,ppRightLowerKneeArray,ppRightLowerKneeNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppKneeSlices-1, 0, ppSegments-1, ppRightLowerKneeArray, ppRightLowerKneeNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
    #local ppRightLowerKnee = ppMeshFromArray(ppRightLowerKneeArray,ppRightLowerKneeNormal,ppKneeSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppRightLowerKneeArray,ppKneeSlices,ppSegments,0.001,<0,1,0>) #end   
  
    #if (ppVerbose = 1) #debug concat("Start of ",ppTrousersOrShorts," Lower Right Leg","\n")  #end 
    #local ppStartRefVector = -VPerp_To_Plane(ppRightKneePos-ppRightHipPos,ppRightAnklePos-ppRightKneePos);  // Doesn't work with straight leg.
    #local ppEndRefVector   = ppStartRefVector;
    ppLimbScan(ppRightCalfObject,ppRightCalfStartPos,ppRightCalfStartVector,ppStartRefVector,ppRightCalfEndPos,ppRightCalfEndVector,ppEndRefVector,ppRightCalfSlices,ppSegments,ppRightCalfArray,ppRightCalfNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppRightCalfSlices-1, 0, ppSegments-1, ppRightCalfArray, ppRightCalfNormal, ppFabricDisplacementPigment,ppFabricBumpiness,ppFabricOffSet)
    #local ppRightCalf = ppMeshFromArray(ppRightCalfArray,ppRightCalfNormal,ppRightCalfSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppRightCalfArray,ppRightCalfSlices,ppSegments,0.001,<0,1,0>) #end   
  #end
  
  
  #if (ppWaistBandOn) 
    #if (ppVerbose = 1) #debug concat("Start of Waist Band","\n")  #end 
    #local ppHipLine = ppLeftHipPos-ppRightHipPos;
    #local ppRefVector = -VPerp_To_Plane(ppHipLine,ppBodyLine);
    // The waist band is generated by scanning the hip object to get half the points. This half is copied in reverse order into the
    // second half, then the first half is offset outwards and the second half inwards.
    #local ppOuterSlices = (ppWaistBandSlices-1)/2;
    ppLimbScan(ppHipObject,ppWaistBandStartPos,ppBodyLine,ppRefVector,ppWaistBandEndPos,ppBodyLine,ppRefVector,ppOuterSlices,ppSegments,ppWaistBandArray,ppWaistBandNormal,0,ppRotationDirection,ppTraceLength)
    ppAddOffSetToArray(0, ppOuterSlices-1, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,ppFabricOffSet)
    ppAddOffSetToArray(0, 0, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,ppFabricOffSet*0.4)
    ppAddOffSetToArray(ppOuterSlices-1, ppOuterSlices-1, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,ppFabricOffSet*0.75)
    #local ppI = 0;
    #while (ppI<ppOuterSlices)
      #local ppJ = 0;
      #while (ppJ<ppSegments)
        #local ppWaistBandArray [ppWaistBandSlices-ppI-2][ppJ] = ppWaistBandArray [ppI][ppJ];
        #local ppWaistBandNormal[ppWaistBandSlices-ppI-2][ppJ] = ppWaistBandNormal[ppI][ppJ];
        #if (ppI = 0)
          #local ppWaistBandArray [ppWaistBandSlices-1][ppJ] = ppWaistBandArray [ppI][ppJ];
          #local ppWaistBandNormal[ppWaistBandSlices-1][ppJ] = ppWaistBandNormal[ppI][ppJ];
        #end
        #local ppJ = ppJ + 1;
      #end
      #local ppI = ppI + 1;
    #end
    ppAddOffSetToArray(ppOuterSlices, ppWaistBandSlices-2, 0, ppSegments-1, ppWaistBandArray, ppWaistBandNormal, ppFabricDisplacementPigment,0,-ppFabricOffSet*1.5)
    #declare ppWaistBand = ppMeshFromArray(ppWaistBandArray,ppWaistBandNormal,ppWaistBandSlices,ppSegments,ppJoinEnds)
    #if (ppShowPoints=1) ppDrawPoints(ppWaistBandArray,ppWaistBandSlices,ppSegments,0.001,<1,2,1>) #end 
  #end
  
  
  
  // Define the profile for a single belt hoop
  #local ppBeltHoopArray[0][0] = <-ppBeltHoopWidth/2,0,0>;
  #local ppBeltHoopArray[1][0] = <-ppBeltHoopWidth/2,-ppBeltHoopHeight*0.15,0>;
  #local ppBeltHoopArray[2][0] = <-ppBeltHoopWidth/2,-ppBeltHoopHeight*0.3,-ppBeltHoopThickness*1.8>;
  #local ppBeltHoopArray[3][0] = <-ppBeltHoopWidth/2,-ppBeltHoopHeight*0.7,-ppBeltHoopThickness*1.5>;
  #local ppBeltHoopArray[4][0] = <-ppBeltHoopWidth/2,-ppBeltHoopHeight*0.85,0>;
  #local ppBeltHoopArray[5][0] = <-ppBeltHoopWidth/2,-ppBeltHoopHeight*1.0,0>;
  #local ppBeltHoopNormal[0][0] = <0,0,1>;
  #local ppBeltHoopNormal[1][0] = <0,-2,1>;
  #local ppBeltHoopNormal[2][0] = <0,-1,1>;
  #local ppBeltHoopNormal[3][0] = <0,0.4,1>;
  #local ppBeltHoopNormal[4][0] = <0,1,1>;
  #local ppBeltHoopNormal[5][0] = <0,0,1>; 
  #local ppI = 0;
  #while (ppI<6)
    #local ppBeltHoopArray[ppI][1] = ppBeltHoopArray[ppI][0]-z*ppBeltHoopThickness; 
    #local ppBeltHoopArray[ppI][2] = ppBeltHoopArray[ppI][1]+x*ppBeltHoopWidth; 
    #local ppBeltHoopArray[ppI][3] = ppBeltHoopArray[ppI][2]+z*ppBeltHoopThickness; 
    #local ppBeltHoopNormal[ppI][1] = -ppBeltHoopNormal[ppI][0]; 
    #local ppBeltHoopNormal[ppI][2] = -ppBeltHoopNormal[ppI][0]; 
    #local ppBeltHoopNormal[ppI][3] =  ppBeltHoopNormal[ppI][0]; 
    #local ppI = ppI + 1;
  #end 
  // Generate a mesh object to represent a single belt hoop
  #declare ppBeltHoopObject = ppMeshFromArray(ppBeltHoopArray,ppBeltHoopNormal,6,3,ppJoinEnds)
   
  // Position a series of copies of the belt hoop object around the waist band of the trousers or shorts
  #if (ppBeltHoopsOn)
    #if (ppVerbose = 1) #debug concat("Start of Belt Hoops","\n")  #end
    #local ppHipLine = ppLeftHipPos-ppRightHipPos;
    #local ppRefVector = -VPerp_To_Plane(ppHipLine,ppBodyLine);
    #local ppSeparationAngle = 360/(ppBeltHoopQuantity+0.4);
    #local ppI = 0;
    #declare ppBeltHoops = union {
      #while (ppI<ppBeltHoopQuantity) 
        #local ppSliceAngle = ppSeparationAngle*ppI + ppSeparationAngle*0.7;
        #local ppNorm = <0, 0, 0>;
        #local ppRayDirection = vnormalize(vaxis_rotate(VPerp_Adjust(ppRefVector, ppBodyLine),ppBodyLine,ppSliceAngle)); 
        #local ppThisPoint = trace(ppHipObject, ppWaistBandEndPos+(ppRayDirection*ppTraceLength), -ppRayDirection, ppNorm);
        #local ppThisPoint = ppThisPoint + ppNorm*ppFabricOffSet*1.5;
        #if (vlength(ppNorm)!=0) 
          // Work out a single set of rotations that will orient the belt hoop as required 
          #local ppXRef = VPerp_To_Plane(ppNorm, ppBodyLine);
          #local ppOrientZ = VRotationD(x,<ppXRef.x,ppXRef.y,0>,z);                           
          #local ppOrientY = VRotationD(x,<vlength(<ppXRef.x,ppXRef.y,0>),0,ppXRef.z>,y);    
          #local ppOrientX = VRotationD(y,vrotate(vrotate(ppBodyLine,<0,0,-ppOrientZ>),<0,-ppOrientY,0>),x);
          object {ppBeltHoopObject rotate <ppOrientX,ppOrientY,ppOrientZ> translate ppThisPoint} 
        #else
          #warning concat("The Belt Hoop placement process sent out a ray from point <",vstr(3,ppSliceCentre,",",0,3),"> that didn't intercept the object.\n")
        #end        
        #local ppI = ppI + 1;
      #end
    }
  #end 
  
  
  // This macro is used to define a set of points that will be used to draw a seam around the end of an item of clothing. 
  // It takes a line of points from a 2D array of points and projects the positions forward in the direction of a given vector. 
  // The points are returned in a pair of 2D target arrays, one for the points one for the surface normals.
  // 
  // Parameters:
  // -----------
  // ppTargetArray   - The 2D array that will contain the positions to be used for the seam
  // ppTargetNormal  - The 2D array that will contain the normals to be used for the seam
  // ppSourceArray   - The 2D array containing the positions of the points on the item of clothing to be used
  // ppSourceNormal  - The 2D array containing the surface normals of the points on the item of clothing to be used
  // ppEndVector     - A 3D vector pointing in the direction of the end of the item of clothing
  // ppI             - The index of the first dimension from the source arrays
  // ppJDim          - The dimension of the second dimension from the source arrays (the number of points to use)
  // ppSeamThickness - A decimal specifying the amount by which the seam protrudes clear of the surface
  // ppSeamWidth     - A decimal specifying the amount by which the seam extends from the end of the clothing
  
  #macro ppEndSeamScan (ppTargetArray,ppTargetNormal,ppSourceArray,ppSourceNormal,ppEndVector,ppI,ppJDim,ppSeamThickness, ppSeamWidth)
    #local ppJ = 0;
    #while (ppJ < ppJDim)
      #local ppTargetArray [0][ppJ] = ppSourceArray [ppI][ppJ];
      #local ppTargetNormal[0][ppJ] = ppSourceNormal[ppI][ppJ];
      #local ppTargetArray [1][ppJ] = ppSourceArray [ppI][ppJ] - ppSeamWidth*vnormalize(ppEndVector);
      #local ppTargetNormal[1][ppJ] = vaxis_rotate(ppSourceNormal[ppI][ppJ],VPerp_To_Plane(ppSourceNormal[ppI][ppJ],ppEndVector),-15);
      #local ppTargetArray [2][ppJ] = ppTargetArray [1][ppJ] - ppSeamThickness*ppSourceNormal[ppI][ppJ];
      #local ppTargetNormal[2][ppJ] = vaxis_rotate(ppSourceNormal[0][ppJ],VPerp_To_Plane(ppSourceNormal[0][ppJ],ppEndVector),-165);
      #local ppJ = ppJ + 1;
    #end
  #end
  
  
  
  #if (ppEndSeamOn) 
  // Generate the Seams around the bottoms of the legs
    #if (ppVerbose = 1) #debug concat("Start of Seams around the bottoms of the legs","\n")  #end
    #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers 
      ppEndSeamScan (ppLeftEndSeamArray ,ppLeftEndSeamNormal ,ppLeftCalfArray ,ppLeftCalfNormal , ppLeftCalfEndVector ,0                  ,ppSegments,ppSeamThickness, ppSeamWidth)
      ppEndSeamScan (ppRightEndSeamArray,ppRightEndSeamNormal,ppRightCalfArray,ppRightCalfNormal,-ppRightCalfEndVector,ppRightCalfSlices-1,ppSegments,ppSeamThickness, ppSeamWidth)
    #else                                                 // If we're doing Shorts
      ppEndSeamScan (ppLeftEndSeamArray ,ppLeftEndSeamNormal ,ppLeftThighArray ,ppLeftThighNormal , ppLeftThighEndVector ,0                   ,ppSegments,ppSeamThickness, ppSeamWidth)
      ppEndSeamScan (ppRightEndSeamArray,ppRightEndSeamNormal,ppRightThighArray,ppRightThighNormal,-ppRightThighEndVector,ppRightThighSlices-1,ppSegments,ppSeamThickness, ppSeamWidth)
    #end
    #declare ppLeftEndSeam  = ppMeshFromArray(ppLeftEndSeamArray, ppLeftEndSeamNormal, ppSeamSlices,ppSegments,ppJoinEnds)
    #declare ppRightEndSeam = ppMeshFromArray(ppRightEndSeamArray,ppRightEndSeamNormal,ppSeamSlices,ppSegments,ppJoinEnds)
  #end  
  
  
  #if (ppSideSeamOn) 
    // Generate the Seams down the sides of the Thighs 
    #if (ppVerbose = 1) #debug concat("Start of Seams down the sides of the Thighs","\n")  #end 
    #local ppJ = (ppSegments/2);  // **** needs rounding ****
    #local ppK = 0;
    #local ppI = 0;
    #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers 
      #local ppK = ppSeamFromArray (ppLeftCalfArray     , ppLeftCalfNormal     , ppLeftCalfSlices, ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppLeftLowerKneeArray, ppLeftLowerKneeNormal, ppKneeSlices    , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppLeftUpperKneeArray, ppLeftUpperKneeNormal, ppKneeSlices    , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #end
    #local ppK = ppSeamFromArray (ppLeftThighArray , ppLeftThighNormal , ppLeftThighSlices , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppLeftHipArray   , ppLeftHipNormal   , ppLeftHipSlices   , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppCrotchArray    , ppCrotchNormal    , ppCrotchSlices    , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppRightHipArray  , ppRightHipNormal  , ppRightHipSlices  , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppRightThighArray, ppRightThighNormal, ppRightThighSlices, ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers 
      #local ppK = ppSeamFromArray (ppRightUpperKneeArray, ppRightUpperKneeNormal, ppKneeSlices     , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppRightLowerKneeArray, ppRightLowerKneeNormal, ppKneeSlices     , ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppRightCalfArray     , ppRightCalfNormal     , ppRightCalfSlices, ppJ, ppSideSeamArray, ppSideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #end
    
    #debug concat("Side Seam ppSideSeamSlices ",str(ppSideSeamSlices,3,3),"\n")
    #debug concat("Side Seam ppK ",str(ppK,3,3),"\n")
  
    
    #declare ppSideSeam  = ppMeshFromArray(ppSideSeamArray, ppSideSeamNormal, ppSeamSlices, ppK,ppDontJoinEnds)
  //  #declare ppSideSeam  = ppMeshFromArray(ppSideSeamArray, ppSideSeamNormal, ppSeamSlices, ppSideSeamSlices,ppDontJoinEnds)
  #end
  
  #if (ppInsideSeamOn) 
    // Generate the seams down the inside of the legs
    #local ppJ = 0;
    #local ppK = 0;
    #local ppI = 0;
    #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers 
      #local ppK = ppSeamFromArray (ppLeftCalfArray     , ppLeftCalfNormal     , ppLeftCalfSlices, ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppLeftLowerKneeArray, ppLeftLowerKneeNormal, ppKneeSlices    , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppLeftUpperKneeArray, ppLeftUpperKneeNormal, ppKneeSlices    , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #end
    #local ppK = ppSeamFromArray (ppLeftThighArray , ppLeftThighNormal , ppLeftThighSlices , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppLeftHipArray   , ppLeftHipNormal   , ppLeftHipSlices   , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppCrotchArray    , ppCrotchNormal    , ppCrotchSlices    , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppRightHipArray  , ppRightHipNormal  , ppRightHipSlices  , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #local ppK = ppSeamFromArray (ppRightThighArray, ppRightThighNormal, ppRightThighSlices, ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #if (strcmp(ppTrousersOrShorts,"Trousers")=0)         // If we're doing Trousers 
      #local ppK = ppSeamFromArray (ppRightUpperKneeArray, ppRightUpperKneeNormal, ppKneeSlices     , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppRightLowerKneeArray, ppRightLowerKneeNormal, ppKneeSlices     , ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
      #local ppK = ppSeamFromArray (ppRightCalfArray     , ppRightCalfNormal     , ppRightCalfSlices, ppJ, ppInsideSeamArray, ppInsideSeamNormal, ppK, ppSeamThickness, ppSeamWidth)
    #end
    #declare ppInsideSeam  = ppMeshFromArray(ppInsideSeamArray, ppInsideSeamNormal, ppSeamSlices, ppK,ppDontJoinEnds)
  #end
  
  #if (ppBackSeamOn) 
  // Generate the seam down the back of the shorts or trousers
    #if (ppVerbose = 1) #debug concat("Start of Seam at back of trousers","\n")  #end
    #local ppI = (ppCrotchSlices-1)/2;
    #local ppJ = ppSegments/2;
    #local ppK = 0;
    #while (ppJ < ppSegments)        // Back points only
      #local ppSeamDisplacement       =  ppSeamWidth*vnormalize(ppCrotchArray [ppI+1][ppJ]-ppCrotchArray [ppI][ppJ]) + ppSeamThickness*VPerp_To_Plane(-ppBodyLine, ppLeftHipPos-ppRightHipPos);
      #local ppBackSeamArray [0][ppK] =  ppCrotchArray [ppI][ppJ];
      #local ppBackSeamNormal[0][ppK] =  ppCrotchNormal[ppI][ppJ];
      #local ppBackSeamArray [1][ppK] =  ppCrotchArray [ppI][ppJ] + ppSeamDisplacement;
      #local ppBackSeamNormal[1][ppK] =  ppCrotchNormal[ppI][ppJ];
      #local ppBackSeamArray [2][ppK] =  ppBackSeamArray [1][ppK] - ppSeamThickness*ppCrotchNormal[ppI][ppJ];
      #local ppBackSeamNormal[2][ppK] = -ppCrotchNormal[ppI][ppJ];                                       
      #local ppK = ppK + 1;
      #local ppJ = ppJ + 1;
    #end 
  //  #declare ppBackSeam  = ppMeshFromArray(ppBackSeamArray, ppBackSeamNormal, ppSeamSlices,ppSegments/2,ppDontJoinEnds)
    #declare ppBackSeam  = ppMeshFromArray(ppBackSeamArray, ppBackSeamNormal, ppSeamSlices,ppK,ppDontJoinEnds)
  #end  
  
  #if (ppZipOn) 
  // Generate the Zip down the front of the shorts or trousers
    #if (ppVerbose = 1) #debug concat("Start of Zip at front of trousers","\n")  #end
    #local ppI = (ppCrotchSlices-1)/2;
    #local ppJ = 0;
    #while (ppJ < ppSegments/2)        // Front points only
      #local ppZipDisplacement   =  ppZipFlapWidth*vnormalize(ppCrotchArray [ppI+1][ppJ]-ppCrotchArray [ppI][ppJ]) + ppZipFlapThickness*VPerp_To_Plane(ppBodyLine, ppLeftHipPos-ppRightHipPos);  
      #local ppZipArray [0][ppJ] =  ppCrotchArray [ppI][ppJ];
      #local ppZipNormal[0][ppJ] =  ppCrotchNormal[ppI][ppJ];
      #local ppZipArray [1][ppJ] =  ppCrotchArray [ppI][ppJ] + ppZipDisplacement;
      #local ppZipNormal[1][ppJ] =  ppCrotchNormal[ppI][ppJ];
      #local ppZipArray [2][ppJ] =  ppZipArray [1][ppJ] - ppZipFlapThickness*ppCrotchNormal[ppI][ppJ];
      #local ppZipNormal[2][ppJ] = -ppCrotchNormal[ppI][ppJ];                                       
      #local ppJ = ppJ + 1;
    #end 
    #declare ppZip  = ppMeshFromArray(ppZipArray, ppZipNormal, ppZipSlices,ppSegments/2,ppDontJoinEnds)
  #end  
  
  
  
  #declare ppLegwear1 = union {
    #if (strcmp(ppTrousersOrShorts,"Trousers")=0)
      object {ppLeftCalf       texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}} 
      object {ppLeftLowerKnee  texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}} 
      object {ppLeftUpperKnee  texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}} 
      object {ppRightLowerKnee texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}} 
      object {ppRightUpperKnee texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}} 
      object {ppRightCalf      texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}}  
    #end
    object {ppLeftThigh    texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}}  
    object {ppLeftHip      texture {ppTrouserLegTexture}}
    object {ppCrotch       texture {ppTrouserLegTexture}}
    object {ppRightHip     texture {ppTrouserLegTexture}}
    object {ppRightThigh   texture {ppTrouserLegTexture}}  // rotate <ppXRot,0,ppZRot>}}  
    #if (ppWaistBandOn) object {ppWaistBand texture {ppTrouserWaistBandTexture}}  #end
    #if (ppEndSeamOn) 
      object {ppLeftEndSeam  texture {ppTrouserSeamTexture}}
      object {ppRightEndSeam texture {ppTrouserSeamTexture}}
    #end
    #if (ppSideSeamOn)   object {ppSideSeam   texture {ppTrouserSeamTexture}} #end
    #if (ppInsideSeamOn) object {ppInsideSeam texture {ppTrouserSeamTexture}} #end
    #if (ppBackSeamOn)   object {ppBackSeam   texture {ppTrouserSeamTexture}} #end
    #if (ppZipOn)        object {ppZip        texture {ppTrouserSeamTexture}} #end
    #if (ppBeltHoopsOn)  object {ppBeltHoops  texture {ppTrouserSeamTexture}} #end
  }           
  
    
  #declare ppLegwearRef = ppAddBodyPart("Trousers",object{ppLegwear1}); 
#end

ppLegwear()